@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -40,7 +40,7 @@
 static void smtp_session_destroy(Session *session);
 
 static gint smtp_auth(SMTPSession *session);
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 static gint smtp_starttls(SMTPSession *session);
 #endif
 static gint smtp_auth_cram_md5(SMTPSession *session);
@@ -80,7 +80,7 @@ Session *smtp_session_new(void)
 
 	session->state                     = SMTP_READY;
 
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 	session->tls_init_done             = FALSE;
 #endif
 
@@ -330,7 +330,7 @@ static gint smtp_ehlo_recv(SMTPSession *session, const gchar *msg)
 	return SM_ERROR;
 }
 
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 static gint smtp_starttls(SMTPSession *session)
 {
 	session->state = SMTP_STARTTLS;
@@ -580,7 +580,7 @@ static gint smtp_session_recv_msg(Session *session, const gchar *msg)
 		if (strstr(msg, "ESMTP"))
 			smtp_session->is_esmtp = TRUE;
 	case SMTP_CONNECTED:
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 		if (smtp_session->user || session->ssl_type != SSL_NONE ||
 		    smtp_session->is_esmtp)
 #else
@@ -608,7 +608,7 @@ static gint smtp_session_recv_msg(Session *session, const gchar *msg)
 			smtp_session->error_val = SM_ERROR;
 			return -1;
 		}
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 		if (session->ssl_type == SSL_STARTTLS &&
 		    smtp_session->tls_init_done == FALSE) {
 			ret = smtp_starttls(smtp_session);
@@ -617,7 +617,7 @@ static gint smtp_session_recv_msg(Session *session, const gchar *msg)
 #endif
 		if (smtp_session->user) {
 			if (smtp_auth(smtp_session) != SM_OK) {
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 				if (session->ssl_type == SSL_NONE
 				&&  smtp_session->tls_init_done == FALSE
 				&&  (smtp_session->avail_auth_type & SMTPAUTH_TLS_AVAILABLE))
@@ -630,7 +630,7 @@ static gint smtp_session_recv_msg(Session *session, const gchar *msg)
 			ret = smtp_from(smtp_session);
 		break;
 	case SMTP_STARTTLS:
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 		if (session_start_tls(session) < 0) {
 			log_warning(LOG_PROTOCOL, _("couldn't start TLS session\n"));
 			smtp_session->state = SMTP_ERROR;
