@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #include "defs.h"
@@ -237,6 +237,9 @@ static void folderview_send_queue_cb	(FolderView	*folderview,
 static void folderview_search_cb	(FolderView	*folderview,
 					 guint		 action,
 					 GtkWidget	*widget);
+static void folderview_run_processing_cb(FolderView	*folderview,
+					 guint		 action,
+					 GtkWidget	*widget);
 
 static void folderview_property_cb	(FolderView	*folderview,
 					 guint		 action,
@@ -260,8 +263,10 @@ static void folderview_drag_received_cb  (GtkWidget        *widget,
 					  guint             info,
 					  guint             time,
 					  FolderView       *folderview);
+#ifndef MAEMO
 static void folderview_start_drag	 (GtkWidget *widget, gint button, GdkEvent *event,
 			                  FolderView       *folderview);
+#endif
 static void folderview_drag_data_get     (GtkWidget        *widget,
 					  GdkDragContext   *drag_context,
 					  GtkSelectionData *selection_data,
@@ -289,6 +294,7 @@ static GtkItemFactoryEntry folderview_common_popup_entries[] =
 {
 	{N_("/Mark all re_ad"),		NULL, mark_all_read_cb, 0, NULL},
 	{"/---",                     	NULL, NULL, 0, "<Separator>"},
+	{N_("/R_un processing rules"),	NULL, folderview_run_processing_cb, 0, NULL},
 	{N_("/_Search folder..."),	NULL, folderview_search_cb, 0, NULL},
 	{N_("/_Properties..."),		NULL, folderview_property_cb, 0, NULL},
 	{N_("/Process_ing..."),		NULL, folderview_processing_cb, 0, NULL},
@@ -373,9 +379,11 @@ static void create_ifactories(gpointer key, gpointer value, gpointer data)
 static void folderview_column_set_titles(FolderView *folderview)
 {
 	GtkWidget *ctree = folderview->ctree;
+	GtkWidget *label_folder;
 	GtkWidget *label_new;
 	GtkWidget *label_unread;
 	GtkWidget *label_total;
+	GtkWidget *hbox_folder;
 	GtkWidget *hbox_new;
 	GtkWidget *hbox_unread;
 	GtkWidget *hbox_total;
@@ -393,18 +401,22 @@ static void folderview_column_set_titles(FolderView *folderview)
 			 &unreadxpm, &unreadxpmmask);
 	stock_pixmap_gdk(ctree, STOCK_PIXMAP_READ,
 			 &readxpm, &readxpmmask);
-		
+	
+	label_folder = gtk_label_new(_("Folder"));
 	label_new = gtk_image_new_from_pixmap(newxpm, newxpmmask);
 	label_unread = gtk_image_new_from_pixmap(unreadxpm, unreadxpmmask);
 	label_total = gtk_image_new_from_pixmap(readxpm, readxpmmask);
 	
 	gtk_clist_column_titles_active(GTK_CLIST(ctree));
 	 
+	hbox_folder = gtk_hbox_new(FALSE, 4);
 	hbox_new = gtk_hbox_new(FALSE, 4);
 	hbox_unread = gtk_hbox_new(FALSE, 4);
 	hbox_total = gtk_hbox_new(FALSE, 4);
 
 	/* left justified */
+	gtk_box_pack_start(GTK_BOX(hbox_folder), label_folder, TRUE, TRUE, 0);
+	gtk_misc_set_alignment (GTK_MISC (label_folder), 0, 0.5);
 	gtk_box_pack_start(GTK_BOX(hbox_new), label_new, TRUE, TRUE, 0);
 	gtk_misc_set_alignment (GTK_MISC (label_new), 1, 0.5);
 	gtk_box_pack_start(GTK_BOX(hbox_unread), label_unread, TRUE, TRUE, 0);
@@ -412,14 +424,26 @@ static void folderview_column_set_titles(FolderView *folderview)
 	gtk_box_pack_start(GTK_BOX(hbox_total), label_total, TRUE, TRUE, 0);
 	gtk_misc_set_alignment (GTK_MISC (label_total), 1, 0.5);
 
+	gtk_widget_show_all(hbox_folder);
 	gtk_widget_show_all(hbox_new);
 	gtk_widget_show_all(hbox_unread);
 	gtk_widget_show_all(hbox_total);
 
+#ifdef MAEMO
+	gtk_widget_set_size_request(hbox_new, -1, 20);
+	gtk_widget_set_size_request(hbox_unread, -1, 20);
+	gtk_widget_set_size_request(hbox_total, -1, 20);
+#endif
+
+	gtk_clist_set_column_widget(GTK_CLIST(ctree),col_pos[F_COL_FOLDER],hbox_folder);
 	gtk_clist_set_column_widget(GTK_CLIST(ctree),col_pos[F_COL_NEW],hbox_new);
 	gtk_clist_set_column_widget(GTK_CLIST(ctree),col_pos[F_COL_UNREAD],hbox_unread);
 	gtk_clist_set_column_widget(GTK_CLIST(ctree),col_pos[F_COL_TOTAL],hbox_total);
 
+#ifdef MAEMO
+	GTK_EVENTS_FLUSH();
+#endif
+
 	gtk_sctree_set_column_tooltip(GTK_SCTREE(ctree), col_pos[F_COL_NEW], _("New"));
 	gtk_sctree_set_column_tooltip(GTK_SCTREE(ctree), col_pos[F_COL_UNREAD], _("Unread"));
 	gtk_sctree_set_column_tooltip(GTK_SCTREE(ctree), col_pos[F_COL_TOTAL], _("Total"));
@@ -475,6 +499,10 @@ static GtkWidget *folderview_ctree_create(FolderView *folderview)
 	ctree = gtk_sctree_new_with_titles(N_FOLDER_COLS, col_pos[F_COL_FOLDER],
 					   titles);
 
+	if (prefs_common.show_col_headers == FALSE)
+		gtk_clist_column_titles_hide(GTK_CLIST(ctree));
+
+
 	gtk_clist_set_selection_mode(GTK_CLIST(ctree), GTK_SELECTION_BROWSE);
 	gtk_clist_set_column_justification(GTK_CLIST(ctree), col_pos[F_COL_NEW],
 					   GTK_JUSTIFY_RIGHT);
@@ -530,8 +558,12 @@ static GtkWidget *folderview_ctree_create(FolderView *folderview)
 			 folderview);
 	g_signal_connect(G_OBJECT(ctree), "tree_select_row",
 			 G_CALLBACK(folderview_selected), folderview);
+#ifndef MAEMO
+	/* drag-n-dropping folders on maemo is impractical as this 
+	 * opens the folder almost everytime */
 	g_signal_connect(G_OBJECT(ctree), "start_drag",
 			 G_CALLBACK(folderview_start_drag), folderview);
+#endif
 	g_signal_connect(G_OBJECT(ctree), "drag_data_get",
 			 G_CALLBACK(folderview_drag_data_get),
 			 folderview);
@@ -571,13 +603,20 @@ static GtkWidget *folderview_ctree_create(FolderView *folderview)
 
 void folderview_set_column_order(FolderView *folderview)
 {
-	GtkWidget *ctree;
+	GtkWidget *ctree = folderview->ctree;
 	FolderItem *item = folderview_get_selected_item(folderview);
+	FolderItem *sel_item = NULL, *op_item = NULL;
 	GtkWidget *scrolledwin = folderview->scrolledwin;
 
+	if (folderview->selected)
+		sel_item = gtk_ctree_node_get_row_data(GTK_CTREE(ctree), folderview->selected);
+	if (folderview->opened)
+		op_item = gtk_ctree_node_get_row_data(GTK_CTREE(ctree), folderview->opened);
+
 	debug_print("recreating tree...\n");
 	gtk_widget_destroy(folderview->ctree);
 
+
 	folderview->ctree = ctree = folderview_ctree_create(folderview);
 	gtk_scrolled_window_set_hadjustment(GTK_SCROLLED_WINDOW(scrolledwin),
 					    GTK_CLIST(ctree)->hadjustment);
@@ -585,6 +624,11 @@ void folderview_set_column_order(FolderView *folderview)
 					    GTK_CLIST(ctree)->vadjustment);
 	gtk_widget_show(ctree);
 	
+	if (sel_item)
+		folderview->selected = gtk_ctree_find_by_row_data(GTK_CTREE(ctree), NULL, sel_item);
+	if (op_item)
+		folderview->opened = gtk_ctree_find_by_row_data(GTK_CTREE(ctree), NULL, op_item);
+
 	folderview_set(folderview);
 	folderview_column_set_titles(folderview);
 
@@ -704,8 +748,19 @@ void folderview_init(FolderView *folderview)
 	if (!bold_style) {
 		gtkut_convert_int_to_gdk_color(prefs_common.color_new, &gdk_color);
 		bold_style = gtk_style_copy(gtk_widget_get_style(ctree));
-		pango_font_description_set_weight
-			(bold_style->font_desc, PANGO_WEIGHT_BOLD);
+		if (prefs_common.derive_from_normal_font || !BOLD_FONT) {
+			pango_font_description_set_weight
+				(bold_style->font_desc, PANGO_WEIGHT_BOLD);
+		} else {
+			PangoFontDescription *font_desc;
+			font_desc = pango_font_description_from_string(BOLD_FONT);
+			if (font_desc) {
+				if (bold_style->font_desc)
+					pango_font_description_free
+						(bold_style->font_desc);
+				bold_style->font_desc = font_desc;
+			}
+		}
 		bold_color_style = gtk_style_copy(bold_style);
 		bold_color_style->fg[GTK_STATE_NORMAL] = gdk_color;
 
@@ -724,7 +779,7 @@ static gboolean folderview_defer_set(gpointer data)
 	if (mainwin->lock_count)
 		return TRUE;
 		
-	printf("doing deferred folderview_set now\n");
+	debug_print("doing deferred folderview_set now\n");
 	folderview_set(folderview);
 
 	folderview->deferred_refresh_id = -1;
@@ -744,7 +799,7 @@ void folderview_set(FolderView *folderview)
 		if (folderview->deferred_refresh_id == -1)
 			folderview->deferred_refresh_id = 
 				g_timeout_add(500, folderview_defer_set, folderview);
-		printf("deferred folderview_set\n");
+		debug_print("deferred folderview_set\n");
 		return;
 	}
 
@@ -824,15 +879,20 @@ static void mark_all_read_cb(FolderView *folderview, guint action,
 			prefs_common.ask_mark_all_read = FALSE;
 	}
 
-	summary_lock(folderview->summaryview);
+	
 	folder_item_update_freeze();
-	if (folderview->summaryview->folder_item == item)
+	if (folderview->summaryview->folder_item != item)
+		summary_lock(folderview->summaryview);
+	else
 		summary_freeze(folderview->summaryview);
+		
 	folderutils_mark_all_read(item);
-	if (folderview->summaryview->folder_item == item)
+	
+	if (folderview->summaryview->folder_item != item)
+		summary_unlock(folderview->summaryview);
+	else
 		summary_thaw(folderview->summaryview);
 	folder_item_update_thaw();
-	summary_unlock(folderview->summaryview);
 }
 
 static void folderview_select_node(FolderView *folderview, GtkCTreeNode *node)
@@ -1100,6 +1160,7 @@ void folderview_rescan_tree(Folder *folder, gboolean rebuild)
 		pos = gtk_scrolled_window_get_vadjustment(
 					GTK_SCROLLED_WINDOW(folderview->scrolledwin));
 		gtk_adjustment_set_value(pos, height);
+		gtk_adjustment_changed(pos);
 	}
 	label_window_destroy(window);
 	inc_unlock();
@@ -1140,6 +1201,7 @@ void folderview_fast_rescan_tree(Folder *folder)
 		pos = gtk_scrolled_window_get_vadjustment(
 					GTK_SCROLLED_WINDOW(folderview->scrolledwin));
 		gtk_adjustment_set_value(pos, height);
+		gtk_adjustment_changed(pos);
 	}
 	label_window_destroy(window);
 	inc_unlock();
@@ -1165,7 +1227,7 @@ gint folderview_check_new(Folder *folder)
 	for (list = folderview_list; list != NULL; list = list->next) {
 		folderview = (FolderView *)list->data;
 		ctree = GTK_CTREE(folderview->ctree);
-
+		folderview->scanning_folder = folder;
 		inc_lock();
 		main_window_lock(folderview->mainwin);
 
@@ -1183,6 +1245,11 @@ gint folderview_check_new(Folder *folder)
 					item->path ? item->path : item->name);
 				continue;
 			}
+			if (item->scanning != ITEM_NOT_SCANNING) {
+				debug_print("skipping %s, scanning\n",
+					item->path ? item->path : item->name);
+				continue;
+			}
 
 			str = get_scan_str(item);
 
@@ -1232,7 +1299,7 @@ gint folderview_check_new(Folder *folder)
 			former_new_msgs += former_new;
 			STATUSBAR_POP(folderview->mainwin);
 		}
-
+		folderview->scanning_folder = NULL;
 		main_window_unlock(folderview->mainwin);
 		inc_unlock();
 	}
@@ -1836,7 +1903,6 @@ static void folderview_append_folder(FolderView *folderview, Folder *folder)
 static void folderview_set_sens_and_popup_menu(FolderView *folderview, gint row, 
 				GdkEventButton *event)
 {
-	GtkCList *clist = GTK_CLIST(folderview->ctree);
 	FolderItem *item;
 	Folder *folder;
 	FolderViewPopup *fpopup;
@@ -1845,10 +1911,7 @@ static void folderview_set_sens_and_popup_menu(FolderView *folderview, gint row,
 	FolderItem *special_trash = NULL, *special_queue = NULL;
 	PrefsAccount *ac;
 
-	if (row > 0)
-		item = gtk_clist_get_row_data(clist, row);
-	else
-		item = folderview_get_selected_item(folderview);
+	item = folderview_get_selected_item(folderview);
 
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
@@ -1895,11 +1958,19 @@ static void folderview_set_sens_and_popup_menu(FolderView *folderview, gint row,
 #define SET_SENS(name, sens) \
 	menu_set_sensitive(fpopup_factory, name, sens)
 
+	if ( FOLDER_TYPE(item->folder) == F_NEWS || FOLDER_TYPE(item->folder) == F_IMAP )
+		SET_SENS("/Download messages", !item->no_select);
+
 	SET_SENS("/Mark all read", item->unread_msgs >= 1);
 	SET_SENS("/Search folder...", item->total_msgs >= 1 && 
 		 folderview->selected == folderview->opened);
-	SET_SENS("/Properties...", TRUE);
-	SET_SENS("/Processing...", item->node->parent != NULL);
+	SET_SENS("/Properties...", !item->no_select);
+
+	SET_SENS("/Run processing rules", item->prefs->processing &&
+		 item->total_msgs >= 1 && !item->processing_pending);
+	SET_SENS("/Processing...", item->node->parent != NULL && 
+		!item->no_select && !item->processing_pending);
+
 	if (item == folder->trash || item == special_trash
 	    || folder_has_parent_of_type(item, F_TRASH)) {
 		GSList *msglist = folder_item_get_msg_list(item);
@@ -1917,8 +1988,6 @@ static void folderview_set_sens_and_popup_menu(FolderView *folderview, gint row,
 	popup = gtk_item_factory_get_widget(fpopup_factory, fpopup->path);
 	gtk_menu_popup(GTK_MENU(popup), NULL, NULL, NULL, NULL,
 		       event->button, event->time);
-
-
 }
 
 static gboolean folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
@@ -1930,7 +1999,7 @@ static gboolean folderview_button_pressed(GtkWidget *ctree, GdkEventButton *even
 	if (!event) return FALSE;
 
 	if (event->button == 1 || event->button == 2) {
-		if (!gtk_ctree_is_hot_spot (GTK_CTREE(clist), event->x, event->y))
+		if (!gtk_sctree_is_hot_spot (GTK_SCTREE(clist), event->x, event->y))
 			folderview->open_folder = TRUE;
 
 	        if (event->type == GDK_2BUTTON_PRESS) {
@@ -1984,8 +2053,13 @@ static gboolean folderview_button_pressed(GtkWidget *ctree, GdkEventButton *even
 static gboolean folderview_button_released(GtkWidget *ctree, GdkEventButton *event,
 					   FolderView *folderview)
 {
+	int row = -1, column = -1;
+
 	if (!event) return FALSE;
 
+	if (!gtk_clist_get_selection_info(GTK_CLIST(ctree), event->x, event->y,
+					  &row, &column))
+		return FALSE;
 	if (event->button == 1 && folderview->open_folder == FALSE &&
 	    folderview->opened != NULL) {
 		gtkut_ctree_set_focus_row(GTK_CTREE(ctree),
@@ -2111,6 +2185,13 @@ static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 		return;
 	}
 	
+	item = gtk_ctree_node_get_row_data(ctree, row);
+	if (!item) {
+		END_TIMING();
+		folderview->open_folder = FALSE;
+		return;
+	}
+
 	if (!can_select || summary_is_locked(folderview->summaryview)) {
 		if (folderview->opened) {
 			gtkut_ctree_set_focus_row(ctree, folderview->opened);
@@ -2125,12 +2206,6 @@ static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 		END_TIMING();
 		return;
 	}
-	item = gtk_ctree_node_get_row_data(ctree, row);
-	if (!item || item->no_select) {
-		END_TIMING();
-		folderview->open_folder = FALSE;
-		return;
-	}
 
 	can_select = FALSE;
 
@@ -2161,6 +2236,7 @@ static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 	}
 
 	/* Open Folder */
+	/* TODO: wwp: avoid displaying (null) in the status bar */
     	buf = g_strdup_printf(_("Opening Folder %s..."), item->path ? 
 					item->path : "(null)");
 	debug_print("%s\n", buf);
@@ -2169,8 +2245,13 @@ static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 
 	main_window_cursor_wait(folderview->mainwin);
 
-	res = folder_item_open(item);
-	if (res == -1) {
+	if (folderview->scanning_folder == item->folder) {
+		res = -2;
+	} else {
+		res = folder_item_open(item);
+	}
+
+	if (res == -1 && item->no_select == FALSE) {
 		main_window_cursor_normal(folderview->mainwin);
 		STATUSBAR_POP(folderview->mainwin);
 
@@ -2180,7 +2261,7 @@ static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 		can_select = TRUE;
 		END_TIMING();
 		return;
-        } else if (res == -2) {
+        } else if (res == -2 && item->no_select == FALSE) {
 		PostponedSelectData *data = g_new0(PostponedSelectData, 1);
 		data->ctree = ctree;
 		data->row = row;
@@ -2407,11 +2488,9 @@ static void folderview_send_queue_cb(FolderView *folderview, guint action,
 			alertpanel_error_log(_("Some errors occurred while "
 					   "sending queued messages."));
 		else {
-			gchar *tmp = g_strdup_printf(_("Some errors occurred "
+			alertpanel_error_log(_("Some errors occurred "
 					"while sending queued messages:\n%s"), errstr);
 			g_free(errstr);
-			alertpanel_error_log(tmp);
-			g_free(tmp);
 		}
 	}
 }
@@ -2422,6 +2501,23 @@ static void folderview_search_cb(FolderView *folderview, guint action,
 	summary_search(folderview->summaryview);
 }
 
+static void folderview_run_processing_cb(FolderView *folderview, guint action,
+				 GtkWidget *widget)
+{
+	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+	FolderItem *item;
+
+	if (!folderview->selected) return;
+
+	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(item->folder != NULL);
+
+	item->processing_pending = TRUE;
+	folder_item_apply_processing(item);
+	item->processing_pending = FALSE;
+}
+
 static void folderview_property_cb(FolderView *folderview, guint action,
 				   GtkWidget *widget)
 {
@@ -2475,7 +2571,7 @@ void folderview_move_folder(FolderView *folderview, FolderItem *from_folder,
 	
 	if (prefs_common.warn_dnd) {
 		buf = g_strdup_printf(copy ? _("Do you really want to copy folder '%s' in '%s' ?"):
-					     _("Do you really want to make folder '%s' a sub-folder of '%s' ?"), 
+					     _("Do you really want to make folder '%s' a subfolder of '%s' ?"), 
 					from_folder->name, to_folder->name);
 		status = alertpanel_full(copy ? _("Copy folder"):_("Move folder"), buf,
 				       	 GTK_STOCK_NO, GTK_STOCK_YES, NULL, TRUE,
@@ -2605,29 +2701,40 @@ void folderview_set_target_folder_color(gint color_op)
 	}
 }
 
-static gchar *last_font = NULL;
+static gchar *last_smallfont = NULL;
+static gchar *last_normalfont = NULL;
+static gchar *last_boldfont = NULL;
 void folderview_reflect_prefs_pixmap_theme(FolderView *folderview)
 {
 	/* force reinit */
-	g_free(last_font);
-	last_font = NULL;
-	
+	g_free(last_smallfont);
+	last_smallfont = NULL;
+	g_free(last_normalfont);
+	last_normalfont = NULL;
+	g_free(last_boldfont);
+	last_boldfont = NULL;
 }
 
 void folderview_reflect_prefs(void)
 {
-	gboolean update_font = TRUE;
+	gboolean update_font = FALSE;
 	FolderView *folderview = mainwindow_get_mainwindow()->folderview;
 	FolderItem *item = folderview_get_selected_item(folderview);
 	GtkAdjustment *pos = gtk_scrolled_window_get_vadjustment(
 				GTK_SCROLLED_WINDOW(folderview->scrolledwin));
 	gint height = pos->value;
 
-	if (last_font && !strcmp(last_font, NORMAL_FONT))
-		update_font = FALSE;
+	if (!last_smallfont || strcmp(last_smallfont, SMALL_FONT) ||
+			!last_normalfont || strcmp(last_normalfont, NORMAL_FONT) ||
+			!last_boldfont || strcmp(last_boldfont, BOLD_FONT))
+		update_font = TRUE;
 
-	g_free(last_font);
-	last_font = g_strdup(NORMAL_FONT);
+	g_free(last_smallfont);
+	last_smallfont = g_strdup(SMALL_FONT);
+	g_free(last_normalfont);
+	last_normalfont = g_strdup(NORMAL_FONT);
+	g_free(last_boldfont);
+	last_boldfont = g_strdup(BOLD_FONT);
 
 	if (update_font) {		
 		normal_style = normal_color_style = bold_style = 
@@ -2659,6 +2766,7 @@ void folderview_reflect_prefs(void)
 	pos = gtk_scrolled_window_get_vadjustment(
 				GTK_SCROLLED_WINDOW(folderview->scrolledwin));
 	gtk_adjustment_set_value(pos, height);
+	gtk_adjustment_changed(pos);
 	gtk_clist_thaw(GTK_CLIST(folderview->ctree));
 }
 
@@ -2700,7 +2808,7 @@ static void drag_state_start(FolderView *folderview, GtkCTreeNode *node, FolderI
 		folderview->drag_item = item;
 	}			 
 }
-
+#ifndef MAEMO
 static void folderview_start_drag(GtkWidget *widget, gint button, GdkEvent *event,
 			          FolderView       *folderview)
 {
@@ -2715,7 +2823,7 @@ static void folderview_start_drag(GtkWidget *widget, gint button, GdkEvent *even
 				 GDK_ACTION_MOVE|GDK_ACTION_COPY|GDK_ACTION_DEFAULT, button, event);
 	gtk_drag_set_icon_default(context);
 }
-
+#endif
 static void folderview_drag_data_get(GtkWidget        *widget,
 				     GdkDragContext   *drag_context,
 				     GtkSelectionData *selection_data,
@@ -2799,16 +2907,20 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 	int height = (int)pos->page_size;
 	int total_height = (int)pos->upper;
 	int vpos = (int) pos->value;
-
+	int offset = prefs_common.show_col_headers ? 24:0;
+	
 	if (gtk_clist_get_selection_info
-		(GTK_CLIST(widget), x - 24, y - 24, &row, &column)) {
+		(GTK_CLIST(widget), x - offset, y - offset, &row, &column)) {
 		GtkWidget *srcwidget;
 
-		if (y > height - 24 && height + vpos < total_height)
-			gtk_adjustment_set_value(pos, (vpos+5 > height ? height : vpos+5));
-
-		if (y < 48 && y > 0)
+		if (y > height - 24 && height + vpos < total_height) {
+			gtk_adjustment_set_value(pos, (vpos+5 > total_height ? total_height : vpos+5));
+			gtk_adjustment_changed(pos);
+		}
+		if (y < 48 && y > 0) {
 			gtk_adjustment_set_value(pos, (vpos-5 < 0 ? 0 : vpos-5));
+			gtk_adjustment_changed(pos);
+		}
 
 		node = gtk_ctree_node_nth(GTK_CTREE(widget), row);
 		item = gtk_ctree_node_get_row_data(GTK_CTREE(widget), node);
@@ -2937,19 +3049,25 @@ static void folderview_drag_received_cb(GtkWidget        *widget,
 	gint row, column;
 	FolderItem *item = NULL, *src_item;
 	GtkCTreeNode *node;
+	int offset = prefs_common.show_col_headers ? 24:0;
 
 	if (info == TARGET_DUMMY) {
 		drag_state_stop(folderview);
 		if ((void *)strstr(data->data, "FROM_OTHER_FOLDER") != (void *)data->data) {
 			/* comes from summaryview */
 			if (gtk_clist_get_selection_info
-				(GTK_CLIST(widget), x - 24, y - 24, &row, &column) == 0)
+				(GTK_CLIST(widget), x - offset, y - offset, &row, &column) == 0)
 				return;
 
 			node = gtk_ctree_node_nth(GTK_CTREE(widget), row);
 			item = gtk_ctree_node_get_row_data(GTK_CTREE(widget), node);
 			src_item = folderview->summaryview->folder_item;
 
+			if (item->no_select) {
+				alertpanel_error(_("The destination folder can only be used to "
+						   "store subfolders."));
+				return;
+			}
 			/* re-check (due to acceptable possibly set for folder moves */
 			if (!(item && item->folder && item->path && !item->no_select && 
 			      src_item && src_item != item && FOLDER_CLASS(item->folder)->copy_msg != NULL)) {
@@ -2980,7 +3098,7 @@ static void folderview_drag_received_cb(GtkWidget        *widget,
 
 			source = data->data + 17;
 			if (gtk_clist_get_selection_info
-			    (GTK_CLIST(widget), x - 24, y - 24, &row, &column) == 0
+			    (GTK_CLIST(widget), x - offset, y - offset, &row, &column) == 0
 			    || *source == 0) {
 				gtk_drag_finish(drag_context, FALSE, FALSE, time);			
 				return;
@@ -2996,8 +3114,7 @@ static void folderview_drag_received_cb(GtkWidget        *widget,
 				!folder_has_parent_of_type(src_item, F_DRAFT) &&
 				!folder_has_parent_of_type(src_item, F_QUEUE) &&
 				!folder_has_parent_of_type(src_item, F_TRASH);
-			if (!item || item->no_select || !src_item 
-			||  !folder_is_normal) {
+			if (!item || !src_item || !folder_is_normal) {
 				gtk_drag_finish(drag_context, FALSE, FALSE, time);			
 				return;
 			}
@@ -3008,7 +3125,7 @@ static void folderview_drag_received_cb(GtkWidget        *widget,
 		folderview->nodes_to_recollapse = NULL;
 	} else if (info == TARGET_MAIL_URI_LIST) {
 		if (gtk_clist_get_selection_info
-			(GTK_CLIST(widget), x - 24, y - 24, &row, &column) == 0)
+			(GTK_CLIST(widget), x - offset, y - offset, &row, &column) == 0)
 			return;
 
 		node = gtk_ctree_node_nth(GTK_CTREE(widget), row);
