@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -51,6 +51,11 @@
 # include <gdk/gdkx.h>
 #endif
 
+#ifdef HAVE_NETWORKMANAGER_SUPPORT
+#include <dbus/dbus-glib.h>
+#include <NetworkManager.h>
+#endif
+
 #include "claws.h"
 #include "main.h"
 #include "mainwindow.h"
@@ -97,14 +102,16 @@
 #include "imap_gtk.h"
 #include "news_gtk.h"
 #include "matcher.h"
+#include "tags.h"
 #ifdef HAVE_LIBETPAN
 #include "imap-thread.h"
+#include "nntp-thread.h"
 #endif
 #include "stock_pixmap.h"
 #ifdef HAVE_VALGRIND
 #include "valgrind.h"
 #endif
-#if USE_OPENSSL
+#if (defined(USE_OPENSSL) || defined (USE_GNUTLS))
 #  include "ssl.h"
 #endif
 
@@ -115,9 +122,18 @@
 #include "timing.h"
 
 #ifdef MAEMO
+#ifdef CHINOOK
+#include <hildon/hildon-banner.h>
+#include <hildon/hildon-program.h>
+#else
+#include <hildon-widgets/hildon-banner.h>
 #include <hildon-widgets/hildon-program.h>
+#endif
 #include <gtk/gtkmain.h>
 #include <libosso.h>
+#include <libgnomevfs/gnome-vfs-volume.h>
+#include <libgnomevfs/gnome-vfs-volume-monitor.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 
 #define OSSO_NAME    "claws_mail"
 #define OSSO_SERVICE "com.nokia."OSSO_NAME
@@ -130,11 +146,22 @@ struct _AppData {
     HildonWindow *window;
     osso_context_t *osso_context;
 };
+
+static GnomeVFSVolumeMonitor *volmon;
+#endif
+
+#ifdef HAVE_NETWORKMANAGER_SUPPORT
+/* Went offline due to NetworkManager */
+static gboolean went_offline_nm;
 #endif
 
 gchar *prog_version;
 gchar *argv0;
 
+#ifdef MAEMO
+HildonProgram *hildon_program;
+#endif
+
 #ifdef HAVE_STARTUP_NOTIFICATION
 static SnLauncheeContext *sn_context = NULL;
 static SnDisplay *sn_display = NULL;
@@ -188,6 +215,11 @@ static void quit_signal_handler         (int sig);
 static void install_basic_sighandlers   (void);
 static void exit_claws			(MainWindow *mainwin);
 
+#ifdef HAVE_NETWORKMANAGER_SUPPORT
+static void networkmanager_state_change_cb(DBusGProxy *proxy, gchar *dev,
+																					 gpointer data);
+#endif
+
 #define MAKE_DIR_IF_NOT_EXIST(dir) \
 { \
 	if (!is_dir_exist(dir)) { \
@@ -206,7 +238,6 @@ static void exit_claws			(MainWindow *mainwin);
 static MainWindow *static_mainwindow;
 
 #ifdef MAEMO
-static HildonProgram *static_hildonprogram;
 static osso_context_t *static_osso_context;
 
 void exit_event_handler(gboolean die_now, gpointer data)
@@ -220,6 +251,19 @@ void exit_event_handler(gboolean die_now, gpointer data)
 
 }
 
+/* Callback for hardware D-BUS events */
+void hw_event_handler(osso_hw_state_t *state, gpointer data)
+{
+	AppData *appdata;
+	appdata = (AppData *) data;
+
+	if (state->shutdown_ind) {
+		exit_claws(static_mainwindow);
+		hildon_banner_show_information(GTK_WIDGET(appdata->window), NULL,
+			_("Exiting..."));
+	}
+}
+
 /* Callback for normal D-BUS messages */
 gint dbus_req_handler(const gchar * interface, const gchar * method,
                       GArray * arguments, gpointer data,
@@ -228,6 +272,10 @@ gint dbus_req_handler(const gchar * interface, const gchar * method,
     AppData *appdata;
     appdata = (AppData *) data;
 
+    if (!strcmp(method, "top_application")) {
+	    osso_rpc_free_val(retval);
+	    return OSSO_OK;
+    }
     osso_system_note_infoprint(appdata->osso_context, method, retval);
     osso_rpc_free_val(retval);
 
@@ -288,6 +336,8 @@ static void claws_gtk_idle(void)
 	g_usleep(50000);
 }
 
+static gboolean sc_starting = FALSE;
+
 static gboolean defer_check_all(void *data)
 {
 	gboolean autochk = GPOINTER_TO_INT(data);
@@ -295,6 +345,11 @@ static gboolean defer_check_all(void *data)
 	inc_all_account_mail(static_mainwindow, autochk, 
 			prefs_common.newmail_notify_manu);
 
+	if (sc_starting) {
+		sc_starting = FALSE;
+		main_window_set_menu_sensitive(static_mainwindow);
+		toolbar_main_set_sensitive(static_mainwindow);
+	}
 	return FALSE;
 }
 
@@ -302,6 +357,11 @@ static gboolean defer_check(void *data)
 {
 	inc_mail(static_mainwindow, prefs_common.newmail_notify_manu);
 
+	if (sc_starting) {
+		sc_starting = FALSE;
+		main_window_set_menu_sensitive(static_mainwindow);
+		toolbar_main_set_sensitive(static_mainwindow);
+	}
 	return FALSE;
 }
 
@@ -315,6 +375,11 @@ static gboolean defer_jump(void *data)
 		defer_check(NULL);
 	} 
 	mainwindow_jump_to(data, FALSE);
+	if (sc_starting) {
+		sc_starting = FALSE;
+		main_window_set_menu_sensitive(static_mainwindow);
+		toolbar_main_set_sensitive(static_mainwindow);
+	}
 	return FALSE;
 }
 
@@ -397,18 +462,20 @@ backup_mode:
 	return (r == 0);
 }
 
-static void migrate_common_rc(const gchar *old_rc, const gchar *new_rc)
+static int migrate_common_rc(const gchar *old_rc, const gchar *new_rc)
 {
 	FILE *oldfp, *newfp;
 	gchar *plugin_path, *old_plugin_path, *new_plugin_path;
 	gchar buf[BUFFSIZE];
+	gboolean err = FALSE;
+
 	oldfp = g_fopen(old_rc, "r");
 	if (!oldfp)
-		return;
+		return -1;
 	newfp = g_fopen(new_rc, "w");
 	if (!newfp) {
 		fclose(oldfp);
-		return;
+		return -1;
 	}
 	
 	plugin_path = g_strdup(get_plugin_dir());
@@ -425,19 +492,22 @@ static void migrate_common_rc(const gchar *old_rc, const gchar *new_rc)
 	debug_print("replacing %s with %s\n", old_plugin_path, new_plugin_path);
 	while (fgets(buf, sizeof(buf), oldfp)) {
 		if (strncmp(buf, old_plugin_path, strlen(old_plugin_path))) {
-			fputs(buf, newfp);
+			err |= (fputs(buf, newfp) == EOF);
 		} else {
 			debug_print("->replacing %s", buf);
 			debug_print("  with %s%s", new_plugin_path, buf+strlen(old_plugin_path));
-			fputs(new_plugin_path, newfp);
-			fputs(buf+strlen(old_plugin_path), newfp);
+			err |= (fputs(new_plugin_path, newfp) == EOF);
+			err |= (fputs(buf+strlen(old_plugin_path), newfp) == EOF);
 		}
 	}
 	g_free(plugin_path);
 	g_free(new_plugin_path);
 	g_free(old_plugin_path);
 	fclose(oldfp);
-	fclose(newfp);
+	if (fclose(newfp) == EOF)
+		err = TRUE;
+	
+	return (err ? -1:0);
 }
 
 #ifdef HAVE_LIBSM
@@ -612,7 +682,6 @@ static void sc_session_manager_connect(MainWindow *mainwin)
 #endif
 
 static gboolean sc_exiting = FALSE;
-static gboolean sc_starting = FALSE;
 static gboolean show_at_startup = TRUE;
 static gboolean claws_crashed_bool = FALSE;
 
@@ -625,11 +694,228 @@ void main_set_show_at_startup(gboolean show)
 	show_at_startup = show;
 }
 
+#ifdef MAEMO
+static void main_vol_mount_cb(GnomeVFSVolumeMonitor *vfs, GnomeVFSVolume *vol, MainWindow *mainwin)
+{
+	gchar *uri = gnome_vfs_volume_get_activation_uri (vol);
+	gchar *mount_path = uri?gnome_vfs_get_local_path_from_uri (uri):NULL;
+	g_free (uri);
+	if (mount_path) {
+		if(!strcmp(mount_path, prefs_common.data_root)) {
+			gtk_widget_set_sensitive(mainwin->window, TRUE);
+			inc_unlock();
+		}
+	}
+	g_free(mount_path);
+}
+static void main_vol_unmount_cb(GnomeVFSVolumeMonitor *vfs, GnomeVFSVolume *vol, MainWindow *mainwin)
+{
+	gchar *uri = gnome_vfs_volume_get_activation_uri (vol);
+	gchar *mount_path = uri?gnome_vfs_get_local_path_from_uri (uri):NULL;
+	g_free (uri);
+	if (mount_path) {
+		if(!strcmp(mount_path, prefs_common.data_root)) {
+			gtk_widget_set_sensitive(mainwin->window, FALSE);
+			inc_lock();
+		}
+	}
+	g_free(mount_path);
+}
+#endif
+
+#ifdef G_OS_WIN32
+static FILE* win32_debug_fp=NULL;
+static guint win32_log_handler_app_id;
+static guint win32_log_handler_glib_id;
+static guint win32_log_handler_gtk_id;
+
+static void win32_print_stdout(const gchar *string)
+{
+	if (win32_debug_fp) {
+		fprintf(win32_debug_fp, string);
+		fflush(win32_debug_fp);
+	}
+}
+
+static void win32_print_stderr(const gchar *string)
+{
+	if (win32_debug_fp) {
+		fprintf(win32_debug_fp, string);
+		fflush(win32_debug_fp);
+	}
+}
+
+static void win32_log(const gchar *log_domain, GLogLevelFlags log_level, const gchar* message, gpointer user_data)
+{
+	if (win32_debug_fp) {
+		const gchar* type;
+
+		switch(log_level & G_LOG_LEVEL_MASK)
+		{
+			case G_LOG_LEVEL_ERROR:
+				type="error";
+				break;
+			case G_LOG_LEVEL_CRITICAL:
+				type="critical";
+				break;
+			case G_LOG_LEVEL_WARNING:
+				type="warning";
+				break;
+			case G_LOG_LEVEL_MESSAGE:
+				type="message";
+				break;
+			case G_LOG_LEVEL_INFO:
+				type="info";
+				break;
+			case G_LOG_LEVEL_DEBUG:
+				type="debug";
+				break;
+			default:
+				type="N/A";
+		}
+		if (log_domain)
+			fprintf(win32_debug_fp, "%s: %s: %s", log_domain, type, message);
+		else
+			fprintf(win32_debug_fp, "%s: %s", type, message);
+		fflush(win32_debug_fp);
+	}
+}
+
+static void win32_open_log(void)
+{
+	if (is_file_exist("claws-win32.log")) {
+		if (rename_force("claws-win32.log", "claws-win32.log.bak") < 0)
+			FILE_OP_ERROR("claws-win32.log", "rename");
+	}
+	win32_debug_fp = fopen("claws-win32.log", "w");
+	if (win32_debug_fp)
+	{
+		g_set_print_handler(win32_print_stdout);
+		g_set_printerr_handler(win32_print_stdout);
+		win32_log_handler_app_id = g_log_set_handler(NULL, G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
+                     | G_LOG_FLAG_RECURSION, win32_log, NULL);
+		win32_log_handler_glib_id = g_log_set_handler("GLib", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
+                     | G_LOG_FLAG_RECURSION, win32_log, NULL);
+		win32_log_handler_gtk_id = g_log_set_handler("Gtk", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
+                     | G_LOG_FLAG_RECURSION, win32_log, NULL);
+	}
+}
+
+static void win32_close_log(void)
+{
+	if (win32_debug_fp)
+	{
+		g_log_remove_handler("", win32_log_handler_app_id);
+		g_log_remove_handler("GLib", win32_log_handler_glib_id);
+		g_log_remove_handler("Gtk", win32_log_handler_gtk_id);
+		fclose(win32_debug_fp);
+		win32_debug_fp=NULL;
+	}
+}		
+#endif
+
+static void main_dump_features_list(gboolean show_debug_only)
+/* display compiled-in features list */
+{
+	if (show_debug_only && !debug_get_mode())
+		return;
+
+	if (show_debug_only)
+		debug_print("GTK+ %d.%d.%d / GLib %d.%d.%d\n",
+			   gtk_major_version, gtk_minor_version, gtk_micro_version,
+			   glib_major_version, glib_minor_version, glib_micro_version);
+	else
+		g_print("GTK+ %d.%d.%d / GLib %d.%d.%d\n",
+			   gtk_major_version, gtk_minor_version, gtk_micro_version,
+			   glib_major_version, glib_minor_version, glib_micro_version);
+	if (show_debug_only)
+		debug_print("Compiled-in features:\n");
+	else
+		g_print("Compiled-in features:\n");
+#if HAVE_LIBCOMPFACE
+	if (show_debug_only)
+		debug_print(" compface\n");
+	else
+		g_print(" compface\n");
+#endif
+#if USE_ASPELL
+	if (show_debug_only)
+		debug_print(" aspell\n");
+	else
+		g_print(" aspell\n");
+#endif
+#if USE_GNUTLS
+	if (show_debug_only)
+		debug_print(" gnutls\n");
+	else
+		g_print(" gnutls\n");
+#endif
+#if INET6
+	if (show_debug_only)
+		debug_print(" ipv6\n");
+	else
+		g_print(" ipv6\n");
+#endif
+#if HAVE_ICONV
+	if (show_debug_only)
+		debug_print(" iconv\n");
+	else
+		g_print(" iconv\n");
+#endif
+#if USE_JPILOT
+	if (show_debug_only)
+		debug_print(" jpilot\n");
+	else
+		g_print(" jpilot\n");
+#endif
+#if USE_LDAP
+	if (show_debug_only)
+		debug_print(" ldap\n");
+	else
+		g_print(" ldap\n");
+#endif
+#if HAVE_LIBETPAN
+	if (show_debug_only)
+		debug_print(" libetpan %d.%d\n", LIBETPAN_VERSION_MAJOR, LIBETPAN_VERSION_MINOR);
+	else
+		g_print(" libetpan %d.%d\n", LIBETPAN_VERSION_MAJOR, LIBETPAN_VERSION_MINOR);
+#endif
+#if USE_GNOMEPRINT
+	if (show_debug_only)
+		debug_print(" gnomeprint\n");
+	else
+		g_print(" gnomeprint\n");
+#endif
+#if HAVE_LIBSM
+	if (show_debug_only)
+		debug_print(" libsm\n");
+	else
+		g_print(" libsm\n");
+#endif
+#if HAVE_NETWORKMANAGER_SUPPORT
+	if (show_debug_only)
+		debug_print(" NetworkManager\n");
+	else
+		g_print(" NetworkManager\n");
+#endif
+#if USE_OPENSSL
+	if (show_debug_only)
+		debug_print(" openssl\n");
+	else
+		g_print(" openssl\n");
+#endif
+}
+
 int main(int argc, char *argv[])
 {
 #ifdef MAEMO
 	osso_context_t *osso_context;
 	osso_return_t result;
+#endif
+#ifdef HAVE_NETWORKMANAGER_SUPPORT
+	DBusGConnection *connection;
+  GError *error;
+  DBusGProxy *proxy;
 #endif
 	gchar *userrc;
 	MainWindow *mainwin;
@@ -638,26 +924,39 @@ int main(int argc, char *argv[])
 	gboolean crash_file_present = FALSE;
 	gint num_folder_class = 0;
 	gboolean asked_for_migration = FALSE;
+	gboolean start_done = TRUE;
 
 	START_TIMING("startup");
 	
 	sc_starting = TRUE;
 
+#ifdef G_OS_WIN32
+	win32_open_log();
+#endif
 	if (!claws_init(&argc, &argv)) {
+#ifdef G_OS_WIN32
+		win32_close_log();
+#endif
 		return 0;
 	}
 
-	prefs_prepare_cache();
+	main_dump_features_list(TRUE);
+
 	prog_version = PROG_VERSION;
 	argv0 = g_strdup(argv[0]);
 
 	parse_cmd_opt(argc, argv);
 
+	prefs_prepare_cache();
+
 #ifdef CRASH_DIALOG
 	if (cmd.crash) {
 		gtk_set_locale();
 		gtk_init(&argc, &argv);
 		crash_main(cmd.crash_params);
+#ifdef G_OS_WIN32
+		win32_close_log();
+#endif
 		return 0;
 	}
 	crash_install_handlers();
@@ -691,12 +990,35 @@ int main(int argc, char *argv[])
 	gtk_set_locale();
 	gtk_init(&argc, &argv);
 
+#ifdef HAVE_NETWORKMANAGER_SUPPORT
+	went_offline_nm = FALSE;
+	error = NULL;
+	proxy = NULL;
+  connection = dbus_g_bus_get(DBUS_BUS_SYSTEM, &error);
+
+  if(!connection) {
+		debug_print("Failed to open connection to system bus: %s\n", error->message);
+		g_error_free(error);
+	}
+	else {
+		proxy = dbus_g_proxy_new_for_name(connection,
+																			"org.freedesktop.NetworkManager",
+																			"/org/freedesktop/NetworkManager",
+																			"org.freedesktop.NetworkManager");
+		dbus_g_proxy_add_signal(proxy,"StateChange", G_TYPE_UINT, G_TYPE_INVALID);
+		dbus_g_proxy_connect_signal(proxy, "StateChange",
+			G_CALLBACK(networkmanager_state_change_cb),
+			NULL,NULL);
+	}
+#endif
+
+
 #ifdef MAEMO
 	osso_context = osso_initialize(OSSO_SERVICE, "2.8.1", TRUE, NULL);
 	if (osso_context == NULL) {
 		return OSSO_ERROR;
 	}
-	static_hildonprogram = HILDON_PROGRAM(hildon_program_get_instance());
+	hildon_program = HILDON_PROGRAM(hildon_program_get_instance());
 	static_osso_context = osso_context;
 #endif	
 	gdk_rgb_init();
@@ -715,6 +1037,9 @@ int main(int argc, char *argv[])
 				   "currently available. This will cause "
 				   "crashes. You need to upgrade GTK+ or "
 				   "recompile Claws Mail."));
+#ifdef G_OS_WIN32
+		win32_close_log();
+#endif
 		exit(1);
 	}
 #else
@@ -724,6 +1049,9 @@ int main(int argc, char *argv[])
 				   "currently available. This will cause "
 				   "crashes. You need to recompile "
 				   "Claws Mail."));
+#ifdef G_OS_WIN32
+		win32_close_log();
+#endif
 		exit(1);
 	}
 #endif	
@@ -737,7 +1065,11 @@ int main(int argc, char *argv[])
 	gtk_rc_parse(userrc);
 	g_free(userrc);
 
+#ifdef G_OS_WIN32
+	CHDIR_EXEC_CODE_RETURN_VAL_IF_FAIL(get_home_dir(), 1, win32_close_log(););
+#else
 	CHDIR_RETURN_VAL_IF_FAIL(get_home_dir(), 1);
+#endif
 	
 	/* no config dir exists. See if we can migrate an old config. */
 	if (!is_dir_exist(RC_DIR)) {
@@ -765,9 +1097,13 @@ int main(int argc, char *argv[])
 		/* If migration failed or the user didn't want to do it,
 		 * we create a new one (and we'll hit wizard later). 
 		 */
-		if (r == FALSE && !is_dir_exist(RC_DIR) && make_dir(RC_DIR) < 0)
+		if (r == FALSE && !is_dir_exist(RC_DIR) && make_dir(RC_DIR) < 0) {
+#ifdef G_OS_WIN32
+			win32_close_log();
+#endif
 			exit(1);
 	}
+	}
 	
 
 	if (!is_file_exist(RC_DIR G_DIR_SEPARATOR_S COMMON_RC) &&
@@ -788,7 +1124,11 @@ int main(int argc, char *argv[])
 	gtk_accel_map_load (userrc);
 	g_free(userrc);
 
+#ifdef G_OS_WIN32
+	CHDIR_EXEC_CODE_RETURN_VAL_IF_FAIL(get_rc_dir(), 1, win32_close_log(););
+#else
 	CHDIR_RETURN_VAL_IF_FAIL(get_rc_dir(), 1);
+#endif
 
 	MAKE_DIR_IF_NOT_EXIST(get_mail_base_dir());
 	MAKE_DIR_IF_NOT_EXIST(get_imap_cache_dir());
@@ -817,7 +1157,11 @@ int main(int argc, char *argv[])
 	}
 	set_log_file(LOG_DEBUG_FILTERING, "filtering.log");
 
+#ifdef G_OS_WIN32
+	CHDIR_EXEC_CODE_RETURN_VAL_IF_FAIL(get_home_dir(), 1, win32_close_log(););
+#else
 	CHDIR_RETURN_VAL_IF_FAIL(get_home_dir(), 1);
+#endif
 
 	folder_system_init();
 	prefs_common_read_config();
@@ -837,20 +1181,17 @@ int main(int argc, char *argv[])
 	prefs_logging_init();
 	prefs_receive_init();
 	prefs_send_init();
+	tags_read_tags();
 #ifdef USE_ASPELL
 	gtkaspell_checkers_init();
 	prefs_spelling_init();
 #endif
 
 	sock_set_io_timeout(prefs_common.io_timeout_secs);
-#ifdef HAVE_LIBETPAN
-	imap_main_set_timeout(prefs_common.io_timeout_secs);
-#endif
 	prefs_actions_read_config();
 	prefs_display_header_read_config();
 	/* prefs_filtering_read_config(); */
 	addressbook_read_file();
-	renderer_read_config();
 
 	gtkut_widget_init();
 	stock_pixbuf_gdk(NULL, STOCK_PIXMAP_CLAWS_MAIL_ICON, &icon);
@@ -863,11 +1204,16 @@ int main(int argc, char *argv[])
 	news_gtk_init();
 
 	mainwin = main_window_create();
+
+#ifdef HAVE_NETWORKMANAGER_SUPPORT
+		networkmanager_state_change_cb(proxy,NULL,mainwin);
+#endif
+
 #ifdef MAEMO
 	AppData *appdata;
 	appdata = g_new0(AppData, 1);
-	appdata->program = static_hildonprogram;
-	appdata->window = mainwin->window;
+	appdata->program = hildon_program;
+	appdata->window = HILDON_WINDOW(mainwin->window);
 	appdata->osso_context = osso_context;
 	result = osso_rpc_set_cb_f(appdata->osso_context, 
                 OSSO_SERVICE, 
@@ -878,6 +1224,7 @@ int main(int argc, char *argv[])
 		return OSSO_ERROR;
 	}
 
+#ifndef CHINOOK
 	/* Add handler for Exit D-BUS messages */
 	result = osso_application_set_exit_cb(appdata->osso_context,
 	                                        exit_event_handler,
@@ -885,6 +1232,9 @@ int main(int argc, char *argv[])
 	if (result != OSSO_OK) {
 		return OSSO_ERROR;
 	}
+#endif
+	osso_hw_set_event_cb( appdata->osso_context,
+				NULL, hw_event_handler, (gpointer) appdata );
 #endif
 	manage_window_focus_in(mainwin->window, NULL, NULL);
 	folderview = mainwin->folderview;
@@ -900,6 +1250,7 @@ int main(int argc, char *argv[])
 					mainwin);
 #endif
 
+
 	prefs_account_init();
 	account_read_config_all();
 
@@ -919,6 +1270,9 @@ int main(int argc, char *argv[])
 		if (!run_wizard(mainwin, TRUE)) {
 			if (asked_for_migration)
 				remove_dir_recursive(RC_DIR);
+#ifdef G_OS_WIN32
+			win32_close_log();
+#endif
 			exit(1);
 		}
 		main_window_reflect_prefs_all_now();
@@ -930,11 +1284,16 @@ int main(int argc, char *argv[])
 		if (!run_wizard(mainwin, FALSE)) {
 			if (asked_for_migration)
 				remove_dir_recursive(RC_DIR);
+#ifdef G_OS_WIN32
+			win32_close_log();
+#endif
 			exit(1);
 		}
-		account_read_config_all();
 		if(!account_get_list()) {
 			exit_claws(mainwin);
+#ifdef G_OS_WIN32
+			win32_close_log();
+#endif
 			exit(1);
 		}
 	}
@@ -950,6 +1309,8 @@ int main(int argc, char *argv[])
 
 #ifdef HAVE_LIBETPAN
 	imap_main_init(prefs_common.skip_ssl_cert_check);
+	imap_main_set_timeout(prefs_common.io_timeout_secs);
+	nntp_main_init(prefs_common.skip_ssl_cert_check);
 #endif	
 	account_set_missing_folder();
 	folder_set_missing_folders();
@@ -1017,7 +1378,7 @@ int main(int argc, char *argv[])
 	}
 
  	plugin_load_standard_plugins ();
-       
+      
 	/* if not crashed, show window now */
 	if (!claws_crashed()) {
 		/* apart if something told not to show */
@@ -1041,18 +1402,52 @@ int main(int argc, char *argv[])
 				   "external plugin. Please reinstall the "
 				   "plugin and try again."));
 			exit_claws(mainwin);
+#ifdef G_OS_WIN32
+			win32_close_log();
+#endif
 			exit(1);
 		}
 	}
 	
 	static_mainwindow = mainwin;
 
+#ifdef MAEMO
+	if (prefs_common.data_root != NULL && *prefs_common.data_root != '\0') {
+		GnomeVFSVolume *vol = NULL;
+		gchar *uri, *mount_path;
+
+		volmon = gnome_vfs_get_volume_monitor();
+		vol = gnome_vfs_volume_monitor_get_volume_for_path(volmon, prefs_common.data_root);
+
+		uri = gnome_vfs_volume_get_activation_uri (vol);
+		mount_path = uri?gnome_vfs_get_local_path_from_uri (uri):NULL;
+		g_free(uri);
+
+		if (vol == NULL || !gnome_vfs_volume_is_mounted(vol) 
+		    || strcmp(mount_path, prefs_common.data_root)) {
+			alertpanel_error(_("Claws Mail can not start without its data volume (%s)."), 
+				prefs_common.data_root);
+			g_free(mount_path);
+			gnome_vfs_volume_unref(vol);
+			exit_claws(mainwin);
+			exit(1);
+		}
+		g_free(mount_path);
+		gnome_vfs_volume_unref(vol);
+		g_signal_connect(G_OBJECT(volmon), 
+				"volume-mounted", G_CALLBACK(main_vol_mount_cb), mainwin);
+		g_signal_connect(G_OBJECT(volmon), 
+				"volume-unmounted", G_CALLBACK(main_vol_unmount_cb), mainwin);
+	}
+#endif
+
 #ifdef HAVE_STARTUP_NOTIFICATION
 	startup_notification_complete(FALSE);
 #endif
 #ifdef HAVE_LIBSM
 	sc_session_manager_connect(mainwin);
 #endif
+
 	folder_item_update_thaw();
 	gtk_clist_thaw(GTK_CLIST(mainwin->folderview->ctree));
 	main_window_cursor_normal(mainwin);
@@ -1064,14 +1459,16 @@ int main(int argc, char *argv[])
 	}
 
 	if (cmd.receive_all && !cmd.target) {
+		start_done = FALSE;
 		g_timeout_add(1000, defer_check_all, GINT_TO_POINTER(FALSE));
 	} else if (prefs_common.chk_on_startup && !cmd.target) {
+		start_done = FALSE;
 		g_timeout_add(1000, defer_check_all, GINT_TO_POINTER(TRUE));
 	} else if (cmd.receive && !cmd.target) {
+		start_done = FALSE;
 		g_timeout_add(1000, defer_check, NULL);
-	} else {
-		gtk_widget_grab_focus(folderview->ctree);
-	}
+	} 
+	gtk_widget_grab_focus(folderview->ctree);
 
 	if (cmd.compose) {
 		open_compose_new(cmd.compose_mailto, cmd.attach_files);
@@ -1090,6 +1487,7 @@ int main(int argc, char *argv[])
 	}
 	
 	if (cmd.target) {
+		start_done = FALSE;
 		g_timeout_add(500, defer_jump, (gpointer)cmd.target);
 	}
 
@@ -1097,7 +1495,11 @@ int main(int argc, char *argv[])
 	
 	compose_reopen_exit_drafts();
 
-	sc_starting = FALSE;
+	if (start_done) {
+		sc_starting = FALSE;
+		main_window_set_menu_sensitive(mainwin);
+		toolbar_main_set_sensitive(mainwin);
+	}
 	END_TIMING();
 
 	gtk_main();
@@ -1105,6 +1507,16 @@ int main(int argc, char *argv[])
 #ifdef MAEMO
 	osso_deinitialize(osso_context);
 #endif
+#ifdef HAVE_NETWORKMANAGER_SUPPORT
+	if(proxy)
+		g_object_unref(proxy);
+	if(connection)
+		dbus_g_connection_unref(connection);
+#endif
+#ifdef G_OS_WIN32
+	win32_close_log();
+#endif
+	utils_free_regex();
 	exit_claws(mainwin);
 
 	return 0;
@@ -1136,6 +1548,11 @@ static void exit_claws(MainWindow *mainwin)
 		main_window_empty_trash(mainwin, prefs_common.ask_on_clean);
 	}
 
+#ifdef HAVE_NETWORKMANAGER_SUPPORT
+	if (prefs_common.work_offline && went_offline_nm)
+		prefs_common.work_offline = FALSE;
+#endif
+
 	/* save prefs for opened folder */
 	if(mainwin->folderview->opened) {
 		FolderItem *item;
@@ -1169,6 +1586,7 @@ static void exit_claws(MainWindow *mainwin)
 
 #ifdef HAVE_LIBETPAN
 	imap_main_done();
+	nntp_main_done();
 #endif
 	/* delete crashfile */
 	if (!cmd.crash)
@@ -1203,7 +1621,9 @@ static void exit_claws(MainWindow *mainwin)
 	prefs_message_done();
 	prefs_other_done();
 	prefs_receive_done();
+	prefs_logging_done();
 	prefs_send_done();
+	tags_write_tags();
 #ifdef USE_ASPELL       
 	prefs_spelling_done();
 	gtkaspell_checkers_quit();
@@ -1267,9 +1687,14 @@ static void parse_cmd_opt(int argc, char *argv[])
 			}
 		} else if (!strncmp(argv[i], "--send", 6)) {
 			cmd.send = TRUE;
+		} else if (!strncmp(argv[i], "--version-full", 14) ||
+			   !strncmp(argv[i], "-V", 2)) {
+			g_print("Claws Mail version " VERSION "\n");
+			main_dump_features_list(FALSE);
+			exit(0);
 		} else if (!strncmp(argv[i], "--version", 9) ||
 			   !strncmp(argv[i], "-v", 2)) {
-			puts("Claws Mail version " VERSION);
+			g_print("Claws Mail version " VERSION "\n");
 			exit(0);
  		} else if (!strncmp(argv[i], "--status-full", 13)) {
  			const gchar *p = argv[i + 1];
@@ -1316,7 +1741,7 @@ static void parse_cmd_opt(int argc, char *argv[])
 			g_print("%s\n", _("  --send                 send all queued messages"));
  			g_print("%s\n", _("  --status [folder]...   show the total number of messages"));
  			g_print("%s\n", _("  --status-full [folder]...\n"
- 			       "                         show the status of each folder"));
+ 			                  "                         show the status of each folder"));
 			g_print("%s\n", _("  --select folder[/msg]  jumps to the specified folder/message\n" 
 			                  "                         folder is a folder id like 'folder/sub_folder'"));
 			g_print("%s\n", _("  --online               switch to online mode"));
@@ -1325,7 +1750,10 @@ static void parse_cmd_opt(int argc, char *argv[])
 			g_print("%s\n", _("  --debug                debug mode"));
 			g_print("%s\n", _("  --help -h              display this help and exit"));
 			g_print("%s\n", _("  --version -v           output version information and exit"));
+			g_print("%s\n", _("  --version-full -V      output version and built-in features information and exit"));
 			g_print("%s\n", _("  --config-dir           output configuration directory"));
+			g_print("%s\n", _("  --alternate-config-dir [dir]\n"
+			                  "                         use specified configuration directory"));
 
 			g_free(base);
 			exit(1);
@@ -1334,8 +1762,10 @@ static void parse_cmd_opt(int argc, char *argv[])
 			cmd.crash_params = g_strdup(argv[i + 1]);
 			i++;
 		} else if (!strncmp(argv[i], "--config-dir", sizeof "--config-dir" - 1)) {
-			puts(RC_DIR);
+			g_print(RC_DIR "\n");
 			exit(0);
+		} else if (!strncmp(argv[i], "--alternate-config-dir", sizeof "--alternate-config-dir" - 1) && i+1 < argc) {
+			set_rc_dir(argv[i+1]);
 		} else if (!strncmp(argv[i], "--exit", 6) ||
 			   !strncmp(argv[i], "--quit", 6) ||
 			   !strncmp(argv[i], "-q", 2)) {
@@ -1356,6 +1786,11 @@ static void parse_cmd_opt(int argc, char *argv[])
 						cmd.compose_mailto = p;
 					}
 				}
+			} else if (!strncmp(argv[i], "file://", 7)) {
+				cmd.target = argv[i];
+			} else if (!strncmp(argv[i], "?attach=file://", strlen("?attach=file://"))) {
+				cmd.compose = TRUE;
+				cmd.compose_mailto = argv[i];
 			} else if (strstr(argv[i], "://")) {
 				const gchar *p = argv[i];
 				if (p && *p != '\0' && *p != '-') {
@@ -1364,6 +1799,8 @@ static void parse_cmd_opt(int argc, char *argv[])
 				}
 			} else if (!strcmp(argv[i], "--sync")) {
 				/* gtk debug */
+			} else if (is_dir_exist(argv[i]) || is_file_exist(argv[i])) {
+				cmd.target = argv[i];
 			} else {
 				g_print(_("Unknown option\n"));
 				exit(1);
@@ -1391,7 +1828,9 @@ static void initial_processing(FolderItem *item, gpointer data)
 
 	
 	if (item->prefs->enable_processing) {
+		item->processing_pending = TRUE;
 		folder_item_apply_processing(item);
+		item->processing_pending = FALSE;
 	}
 
 	STATUSBAR_POP(mainwin);
@@ -1407,7 +1846,7 @@ static gboolean draft_all_messages(void)
 		Compose *c = (Compose*)compose_list->data;
 		if (!compose_draft(c, COMPOSE_DRAFT_FOR_EXIT))
 			return FALSE;
-		compose_list = compose_list->next;
+		compose_list = compose_get_compose_list();
 	}
 	return TRUE;
 }
@@ -1502,10 +1941,15 @@ gboolean claws_is_starting(void)
 gchar *claws_get_socket_name(void)
 {
 	static gchar *filename = NULL;
-
+	const gchar *socket_dir = NULL;
+	
+	if (rc_dir_is_alt())
+		socket_dir = get_rc_dir();
+	else
+		socket_dir = g_get_tmp_dir();
 	if (filename == NULL) {
 		filename = g_strdup_printf("%s%cclaws-mail-%d",
-					   g_get_tmp_dir(), G_DIR_SEPARATOR,
+					   socket_dir, G_DIR_SEPARATOR,
 #if HAVE_GETUID
 					   getuid());
 #else
@@ -1630,7 +2074,7 @@ static gint prohibit_duplicate_launch(void)
 		memset(buf, 0, sizeof(buf));
 		fd_gets(uxsock, buf, sizeof(buf));
 		if (strcmp2(buf, x_display)) {
-			printf("Claws Mail is already running on display %s.\n",
+			g_print("Claws Mail is already running on display %s.\n",
 				buf);
 		} else {
 			fd_close(uxsock);
@@ -1793,11 +2237,9 @@ static void send_queue(void)
 		}
 	}
 	if (errstr) {
-		gchar *tmp = g_strdup_printf(_("Some errors occurred "
+		alertpanel_error_log(_("Some errors occurred "
 				"while sending queued messages:\n%s"), errstr);
 		g_free(errstr);
-		alertpanel_error_log(tmp);
-		g_free(tmp);
 	} else if (error) {
 		alertpanel_error_log("Some errors occurred "
 				"while sending queued messages.");
@@ -1848,12 +2290,104 @@ static void install_basic_sighandlers()
 }
 
 #ifdef MAEMO
-HildonProgram *hildon_program(void)
-{
-	return static_hildonprogram;
-}
 osso_context_t *get_osso_context(void)
 {
 	return static_osso_context;
 }
 #endif
+
+
+#ifdef HAVE_NETWORKMANAGER_SUPPORT
+static void networkmanager_state_change_cb(DBusGProxy *proxy, gchar *dev,
+					 gpointer data)
+{
+	MainWindow *mainWin;
+
+	mainWin = NULL;
+	if (static_mainwindow)
+		mainWin = static_mainwindow;
+	else if (data)
+		mainWin = (MainWindow*)data;
+	
+	if (!prefs_common.use_networkmanager)
+		return;
+
+	if (mainWin) {
+		GError *error;
+		gboolean online;
+
+		error = NULL;		
+		online = networkmanager_is_online(&error);
+		if(!error) {
+			if(online && went_offline_nm) {
+				went_offline_nm = FALSE;
+				main_window_toggle_work_offline(mainWin, FALSE, FALSE);
+				debug_print("NetworkManager: Went online\n");
+			}
+			else if(!online) {
+				went_offline_nm = TRUE;
+				main_window_toggle_work_offline(mainWin, TRUE, FALSE);
+				debug_print("NetworkManager: Went offline\n");
+			}
+		}
+		else {
+			debug_print("Failed to get online information from NetworkManager: %s\n",
+							 error->message);
+			g_error_free(error);
+		}
+	}
+	else
+		debug_print("NetworkManager: Cannot change connection state because "
+						 "main window does not exist\n");
+}
+
+/* Returns true (and sets error appropriately, if given) in case of error */
+gboolean networkmanager_is_online(GError **error)
+{
+	DBusGConnection *connection;
+	DBusGProxy *proxy;
+	GError *tmp_error;
+	gboolean retVal;
+	guint32 state;
+
+	if (!prefs_common.use_networkmanager)
+		return TRUE;
+
+	tmp_error = NULL;
+	proxy = NULL;
+	connection = dbus_g_bus_get(DBUS_BUS_SYSTEM, &tmp_error);
+
+	if(!connection) {
+		/* If calling code doesn't do error checking, at least print some debug */
+		if((error == NULL) || (*error == NULL))
+			debug_print("Failed to open connection to system bus: %s\n",
+							 tmp_error->message);
+		g_propagate_error(error, tmp_error);
+		return TRUE;
+	}
+
+	proxy = dbus_g_proxy_new_for_name(connection,
+			"org.freedesktop.NetworkManager",
+			"/org/freedesktop/NetworkManager",
+			"org.freedesktop.NetworkManager");
+
+	retVal = dbus_g_proxy_call(proxy,"state",&tmp_error, G_TYPE_INVALID,
+			G_TYPE_UINT, &state, G_TYPE_INVALID);
+
+	if(proxy)
+		g_object_unref(proxy);
+	if(connection)
+		dbus_g_connection_unref(connection);
+
+	if(!retVal) {
+		/* If calling code doesn't do error checking, at least print some debug */
+		if((error == NULL) || (*error == NULL))
+			debug_print("Failed to get state info from NetworkManager: %s\n",
+							 tmp_error->message);
+		g_propagate_error(error, tmp_error);
+		return TRUE;
+	}
+
+	return (state == NM_STATE_CONNECTED || state == NM_STATE_UNKNOWN);
+}
+#endif
