@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -45,6 +45,7 @@
 #include <ctype.h>
 #include "prefs_common.h"
 #include "log.h"
+#include "tags.h"
 
 /*!
  *\brief	Keyword lookup element
@@ -80,6 +81,8 @@ static const MatchParser matchparser_tab[] = {
 	{MATCHCRITERIA_NOT_COLORLABEL, "~colorlabel"},
 	{MATCHCRITERIA_IGNORE_THREAD, "ignore_thread"},
 	{MATCHCRITERIA_NOT_IGNORE_THREAD, "~ignore_thread"},
+	{MATCHCRITERIA_WATCH_THREAD, "watch_thread"},
+	{MATCHCRITERIA_NOT_WATCH_THREAD, "~watch_thread"},
 	{MATCHCRITERIA_SPAM, "spam"},
 	{MATCHCRITERIA_NOT_SPAM, "~spam"},
 
@@ -94,6 +97,10 @@ static const MatchParser matchparser_tab[] = {
 	{MATCHCRITERIA_NOT_CC, "~cc"},
 	{MATCHCRITERIA_TO_OR_CC, "to_or_cc"},
 	{MATCHCRITERIA_NOT_TO_AND_NOT_CC, "~to_or_cc"},
+	{MATCHCRITERIA_TAG, "tag"},
+	{MATCHCRITERIA_NOT_TAG, "~tag"},
+	{MATCHCRITERIA_TAGGED, "tagged"},
+	{MATCHCRITERIA_NOT_TAGGED, "~tagged"},
 	{MATCHCRITERIA_AGE_GREATER, "age_greater"},
 	{MATCHCRITERIA_AGE_LOWER, "age_lower"},
 	{MATCHCRITERIA_NEWSGROUPS, "newsgroups"},
@@ -155,7 +162,11 @@ static const MatchParser matchparser_tab[] = {
 	{MATCHACTION_STOP, "stop"},
 	{MATCHACTION_HIDE, "hide"},
 	{MATCHACTION_IGNORE, "ignore"},
-	{MATCHACTION_ADD_TO_ADDRESSBOOK, "add_to_addressbook"}
+	{MATCHACTION_WATCH, "watch"},
+	{MATCHACTION_ADD_TO_ADDRESSBOOK, "add_to_addressbook"},
+	{MATCHACTION_SET_TAG, "set_tag"},
+	{MATCHACTION_UNSET_TAG, "unset_tag"},
+	{MATCHACTION_CLEAR_TAGS, "clear_tags"},
 };
 
 enum {
@@ -245,7 +256,44 @@ MatcherProp *matcherprop_new(gint criteria, const gchar *header,
  	prop = g_new0(MatcherProp, 1);
 	prop->criteria = criteria;
 	prop->header = header != NULL ? g_strdup(header) : NULL;
+
 	prop->expr = expr != NULL ? g_strdup(expr) : NULL;
+
+	prop->matchtype = matchtype;
+	prop->preg = NULL;
+	prop->value = value;
+	prop->error = 0;
+
+	return prop;
+}
+
+/*!
+ *\brief	Allocate a structure for a filtering / scoring
+ *		"condition" (a matcher structure)
+ *		Same as matcherprop_new, except it doesn't change the expr's 
+ *		case.
+ *
+ *\param	criteria Criteria ID (MATCHCRITERIA_XXXX)
+ *\param	header Header string (if criteria is MATCHCRITERIA_HEADER
+			or MATCHCRITERIA_FOUND_IN_ADDRESSBOOK)
+ *\param	matchtype Type of action (MATCHTYPE_XXX)
+ *\param	expr String value or expression to check
+ *\param	value Integer value to check
+ *
+ *\return	MatcherProp * Pointer to newly allocated structure
+ */
+MatcherProp *matcherprop_new_create(gint criteria, const gchar *header,
+			      gint matchtype, const gchar *expr,
+			      int value)
+{
+	MatcherProp *prop;
+
+ 	prop = g_new0(MatcherProp, 1);
+	prop->criteria = criteria;
+	prop->header = header != NULL ? g_strdup(header) : NULL;
+
+	prop->expr = expr != NULL ? g_strdup(expr) : NULL;
+
 	prop->matchtype = matchtype;
 	prop->preg = NULL;
 	prop->value = value;
@@ -306,7 +354,7 @@ static gboolean match_with_addresses_in_addressbook
 	g_return_val_if_fail(address_list != NULL, FALSE);
 
 	debug_print("match_with_addresses_in_addressbook(%d, %s)\n",
-				g_slist_length(address_list), folderpath);
+				g_slist_length(address_list), folderpath?folderpath:"(null)");
 
 	if (folderpath == NULL ||
 		strcasecmp(folderpath, _("Any")) == 0 ||
@@ -392,22 +440,33 @@ static gboolean match_with_addresses_in_addressbook
  *		matcher structure
  */
 static gboolean matcherprop_string_match(MatcherProp *prop, const gchar *str,
-										const gchar *debug_context)
+					 const gchar *debug_context)
 {
 	gchar *str1;
-	gchar *str2;
+	gchar *down_expr;
 	gboolean ret = FALSE;
-
+	gboolean should_free = FALSE;
 	if (str == NULL)
 		return FALSE;
 
+	if (prop->matchtype == MATCHTYPE_REGEXPCASE ||
+	    prop->matchtype == MATCHTYPE_MATCHCASE) {
+		str1 = g_utf8_casefold(str, -1);
+		down_expr = g_utf8_casefold(prop->expr, -1);
+		should_free = TRUE;
+	} else {
+		str1 = (gchar *)str;
+		down_expr = (gchar *)prop->expr;
+		should_free = FALSE;
+	}
+
 	switch (prop->matchtype) {
 	case MATCHTYPE_REGEXPCASE:
 	case MATCHTYPE_REGEXP:
 		if (!prop->preg && (prop->error == 0)) {
 			prop->preg = g_new0(regex_t, 1);
 			/* if regexp then don't use the escaped string */
-			if (regcomp(prop->preg, prop->expr,
+			if (regcomp(prop->preg, down_expr,
 				    REG_NOSUB | REG_EXTENDED
 				    | ((prop->matchtype == MATCHTYPE_REGEXPCASE)
 				    ? REG_ICASE : 0)) != 0) {
@@ -416,10 +475,12 @@ static gboolean matcherprop_string_match(MatcherProp *prop, const gchar *str,
 				prop->preg = NULL;
 			}
 		}
-		if (prop->preg == NULL)
-			return FALSE;
+		if (prop->preg == NULL) {
+			ret = FALSE;
+			goto free_strs;
+		}
 		
-		if (regexec(prop->preg, str, 0, NULL, 0) == 0)
+		if (regexec(prop->preg, str1, 0, NULL, 0) == 0)
 			ret = TRUE;
 		else
 			ret = FALSE;
@@ -432,48 +493,22 @@ static gboolean matcherprop_string_match(MatcherProp *prop, const gchar *str,
 			strretchomp(stripped);
 			if (ret) {
 				log_print(LOG_DEBUG_FILTERING,
-						"%s value [ %s ] matches regular expression [ %s ]\n",
-						debug_context, stripped, prop->expr);
+						"%s value [ %s ] matches regular expression [ %s ] (%s)\n",
+						debug_context, stripped, prop->expr,
+						prop->matchtype == MATCHTYPE_REGEXP ? _("Case sensitive"):_("Case insensitive"));
 			} else {
 				log_print(LOG_DEBUG_FILTERING,
-						"%s value [ %s ] doesn't matches regular expression [ %s ]\n",
-						debug_context, stripped, prop->expr);
+						"%s value [ %s ] doesn't matches regular expression [ %s ] (%s)\n",
+						debug_context, stripped, prop->expr,
+						prop->matchtype == MATCHTYPE_REGEXP ? _("Case sensitive"):_("Case insensitive"));
 			}
 			g_free(stripped);
 		}
 		break;
 			
-	case MATCHTYPE_MATCH:
-		ret = (strstr(str, prop->expr) != NULL);
-
-		/* debug output */
-		if (debug_filtering_session
-				&& prefs_common.filtering_debug_level >= FILTERING_DEBUG_LEVEL_HIGH) {
-			gchar *stripped = g_strdup(str);
-
-			strretchomp(stripped);
-			if (ret) {
-				log_print(LOG_DEBUG_FILTERING,
-						"%s value [ %s ] contains [ %s ] (case sensitive)\n",
-						debug_context, stripped, prop->expr);
-			} else {
-				log_print(LOG_DEBUG_FILTERING,
-						"%s value [ %s ] doesn't contains [ %s ] (case sensitive)\n",
-						debug_context, stripped, prop->expr);
-			}
-			g_free(stripped);
-		}
-		break;
-
-	/* FIXME: put upper in unesc_str */
 	case MATCHTYPE_MATCHCASE:
-		str2 = alloca(strlen(prop->expr) + 1);
-		strcpy(str2, prop->expr);
-		g_strup(str2);
-		str1 = alloca(strlen(str) + 1);
-		strcpy(str1, str);
-		g_strup(str1);
-		ret = (strstr(str1, str2) != NULL);
+	case MATCHTYPE_MATCH:
+		ret = (strstr(str1, down_expr) != NULL);
 
 		/* debug output */
 		if (debug_filtering_session
@@ -483,20 +518,58 @@ static gboolean matcherprop_string_match(MatcherProp *prop, const gchar *str,
 			strretchomp(stripped);
 			if (ret) {
 				log_print(LOG_DEBUG_FILTERING,
-						"%s value [ %s ] contains [ %s ] (case insensitive)\n",
-						debug_context, stripped, prop->expr);
+						"%s value [ %s ] contains [ %s ] (%s)\n",
+						debug_context, stripped, prop->expr,
+						prop->matchtype == MATCHTYPE_MATCH ? _("Case sensitive"):_("Case insensitive"));
 			} else {
 				log_print(LOG_DEBUG_FILTERING,
-						"%s [ %s ] doesn't contains [ %s ] (case insensitive)\n",
-						debug_context, stripped, prop->expr);
+						"%s value [ %s ] doesn't contains [ %s ] (%s)\n",
+						debug_context, stripped, prop->expr,
+						prop->matchtype == MATCHTYPE_MATCH ? _("Case sensitive"):_("Case insensitive"));
 			}
 			g_free(stripped);
 		}
 		break;
-		
+
 	default:
 		break;
 	}
+	
+free_strs:
+	if (should_free) {
+		g_free(str1);
+		g_free(down_expr);
+	}
+	return ret;
+}
+
+/*!
+ *\brief	Find out if a tag matches a condition
+ *
+ *\param	prop Matcher structure
+ *\param	msginfo message to check
+ *
+ *\return	gboolean TRUE if msginfo matches the condition in the 
+ *		matcher structure
+ */
+static gboolean matcherprop_tag_match(MatcherProp *prop, MsgInfo *msginfo,
+					 const gchar *debug_context)
+{
+	gboolean ret = FALSE;
+	GSList *cur;
+
+	if (msginfo == NULL || msginfo->tags == NULL)
+		return FALSE;
+
+	for (cur = msginfo->tags; cur; cur = cur->next) {
+		const gchar *str = tags_get_tag(GPOINTER_TO_INT(cur->data));
+		if (!str)
+			continue;
+		if (matcherprop_string_match(prop, str, debug_context)) {
+			ret = TRUE;
+			break;
+		}
+	}
 	return ret;
 }
 
@@ -583,7 +656,6 @@ static void *matcher_test_thread(void *data)
 	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
 
 	result = system(td->cmd);
-	if (result) perror("system");
 	td->done = TRUE; /* let the caller thread join() */
 	return GINT_TO_POINTER(result);
 }
@@ -605,6 +677,7 @@ static gboolean matcherprop_match_test(const MatcherProp *prop,
 	gint retval;
 #ifdef USE_PTHREAD
 	pthread_t pt;
+	pthread_attr_t pta;
 	thread_data *td = g_new0(thread_data, 1);
 	void *res = NULL;
 	time_t start_time = time(NULL);
@@ -619,7 +692,7 @@ static gboolean matcherprop_match_test(const MatcherProp *prop,
 	if (cmd == NULL)
 		return FALSE;
 
-#if (defined USE_PTHREAD && defined __GLIBC__ && (__GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ >= 3)))
+#ifdef USE_PTHREAD
 	/* debug output */
 	if (debug_filtering_session
 			&& prefs_common.filtering_debug_level >= FILTERING_DEBUG_LEVEL_HIGH) {
@@ -630,11 +703,12 @@ static gboolean matcherprop_match_test(const MatcherProp *prop,
 
 	td->cmd = cmd;
 	td->done = FALSE;
-	if (pthread_create(&pt, PTHREAD_CREATE_JOINABLE, 
-			matcher_test_thread, td) != 0)
+	if (pthread_attr_init(&pta) != 0 ||
+	    pthread_attr_setdetachstate(&pta, PTHREAD_CREATE_JOINABLE) != 0 ||
+	    pthread_create(&pt, &pta, matcher_test_thread, td) != 0)
 		retval = system(cmd);
 	else {
-		printf("waiting for test thread\n");
+		debug_print("waiting for test thread\n");
 		while(!td->done) {
 			/* don't let the interface freeze while waiting */
 			claws_do_idle();
@@ -646,7 +720,7 @@ static gboolean matcherprop_match_test(const MatcherProp *prop,
 		}
 		pthread_join(pt, &res);
 		retval = GPOINTER_TO_INT(res);
-		printf(" test thread returned %d\n", retval);
+		debug_print(" test thread returned %d\n", retval);
 	}
 	g_free(td);
 #else
@@ -767,6 +841,10 @@ gboolean matcherprop_match(MatcherProp *prop,
 		return MSG_IS_IGNORE_THREAD(info->flags);
 	case MATCHCRITERIA_NOT_IGNORE_THREAD:
 		return !MSG_IS_IGNORE_THREAD(info->flags);
+	case MATCHCRITERIA_WATCH_THREAD:
+		return MSG_IS_WATCH_THREAD(info->flags);
+	case MATCHCRITERIA_NOT_WATCH_THREAD:
+		return !MSG_IS_WATCH_THREAD(info->flags);
 	case MATCHCRITERIA_SUBJECT:
 		return matcherprop_string_match(prop, info->subject,
 						prefs_common_translated_header_name("Subject:"));
@@ -832,6 +910,22 @@ gboolean matcherprop_match(MatcherProp *prop,
 		g_free(context2);
 		return ret;
 	}
+	case MATCHCRITERIA_TAG:
+	case MATCHCRITERIA_NOT_TAG:
+	{
+		gboolean ret;
+
+		ret = matcherprop_tag_match(prop, info, _("Tag"));
+		return (prop->criteria == MATCHCRITERIA_TAG)? ret : !ret;
+	}
+	case MATCHCRITERIA_TAGGED:
+	case MATCHCRITERIA_NOT_TAGGED:
+	{
+		gboolean ret;
+
+		ret = (info->tags != NULL);
+		return (prop->criteria == MATCHCRITERIA_TAGGED)? ret : !ret;
+	}
 	case MATCHCRITERIA_AGE_GREATER:
 	{
 		gboolean ret;
@@ -1594,6 +1688,8 @@ gboolean matcherlist_match(MatcherList *matchers, MsgInfo *info)
 		case MATCHCRITERIA_NOT_COLORLABEL:
 		case MATCHCRITERIA_IGNORE_THREAD:
 		case MATCHCRITERIA_NOT_IGNORE_THREAD:
+		case MATCHCRITERIA_WATCH_THREAD:
+		case MATCHCRITERIA_NOT_WATCH_THREAD:
 		case MATCHCRITERIA_SUBJECT:
 		case MATCHCRITERIA_NOT_SUBJECT:
 		case MATCHCRITERIA_FROM:
@@ -1604,6 +1700,10 @@ gboolean matcherlist_match(MatcherList *matchers, MsgInfo *info)
 		case MATCHCRITERIA_NOT_CC:
 		case MATCHCRITERIA_TO_OR_CC:
 		case MATCHCRITERIA_NOT_TO_AND_NOT_CC:
+		case MATCHCRITERIA_TAG:
+		case MATCHCRITERIA_NOT_TAG:
+		case MATCHCRITERIA_TAGGED:
+		case MATCHCRITERIA_NOT_TAGGED:
 		case MATCHCRITERIA_AGE_GREATER:
 		case MATCHCRITERIA_AGE_LOWER:
 		case MATCHCRITERIA_NEWSGROUPS:
@@ -1783,6 +1883,10 @@ gchar *matcherprop_to_string(MatcherProp *matcher)
 	case MATCHCRITERIA_NOT_PARTIAL:
 	case MATCHCRITERIA_IGNORE_THREAD:
 	case MATCHCRITERIA_NOT_IGNORE_THREAD:
+	case MATCHCRITERIA_WATCH_THREAD:
+	case MATCHCRITERIA_NOT_WATCH_THREAD:
+	case MATCHCRITERIA_TAGGED:
+	case MATCHCRITERIA_NOT_TAGGED:
 		return g_strdup(criteria_str);
 	case MATCHCRITERIA_TEST:
 	case MATCHCRITERIA_NOT_TEST:
@@ -2054,7 +2158,7 @@ gchar *matching_build_command(const gchar *cmd, MsgInfo *info)
  *\param	fp File
  *\param	prefs_filtering List of filtering conditions
  */
-static void prefs_filtering_write(FILE *fp, GSList *prefs_filtering)
+static int prefs_filtering_write(FILE *fp, GSList *prefs_filtering)
 {
 	GSList *cur = NULL;
 
@@ -2072,19 +2176,19 @@ static void prefs_filtering_write(FILE *fp, GSList *prefs_filtering)
 		if (prop->enabled) {
 			if (fputs("enabled ", fp) == EOF) {
 				FILE_OP_ERROR("filtering config", "fputs");
-				return;
+				return -1;
 			}
 		} else {
 			if (fputs("disabled ", fp) == EOF) {
 				FILE_OP_ERROR("filtering config", "fputs");
-				return;
+				return -1;
 			}
 		}
 
 		if (fputs("rulename \"", fp) == EOF) {
 			FILE_OP_ERROR("filtering config", "fputs");
 			g_free(filtering_str);
-			return;
+			return -1;
 		}
 		tmp_name = prop->name;
 		while (tmp_name && *tmp_name != '\0') {
@@ -2092,14 +2196,14 @@ static void prefs_filtering_write(FILE *fp, GSList *prefs_filtering)
 				if (fputc(*tmp_name, fp) == EOF) {
 					FILE_OP_ERROR("filtering config", "fputs || fputc");
 					g_free(filtering_str);
-					return;
+					return -1;
 				}
 			} else if (*tmp_name == '"') {
 				if (fputc('\\', fp) == EOF ||
 				    fputc('"', fp) == EOF) {
 					FILE_OP_ERROR("filtering config", "fputs || fputc");
 					g_free(filtering_str);
-					return;
+					return -1;
 				}
 			}
 			tmp_name ++;
@@ -2107,7 +2211,7 @@ static void prefs_filtering_write(FILE *fp, GSList *prefs_filtering)
 		if (fputs("\" ", fp) == EOF) {
 			FILE_OP_ERROR("filtering config", "fputs");
 			g_free(filtering_str);
-			return;
+			return -1;
 		}
 
 		if (prop->account_id != 0) {
@@ -2117,7 +2221,7 @@ static void prefs_filtering_write(FILE *fp, GSList *prefs_filtering)
 			if (fputs(tmp, fp) == EOF) {
 				FILE_OP_ERROR("filtering config", "fputs");
 				g_free(tmp);
-				return;
+				return -1;
 			}
 			g_free(tmp);
 		}
@@ -2126,12 +2230,19 @@ static void prefs_filtering_write(FILE *fp, GSList *prefs_filtering)
 		    fputc('\n', fp) == EOF) {
 			FILE_OP_ERROR("filtering config", "fputs || fputc");
 			g_free(filtering_str);
-			return;
+			return -1;
 		}
 		g_free(filtering_str);
 	}
+	
+	return 0;
 }
 
+typedef struct _NodeLoopData {
+	FILE *fp;
+	gboolean error;
+} NodeLoopData;
+
 /*!
  *\brief	Write matchers from a folder item
  *
@@ -2140,10 +2251,10 @@ static void prefs_filtering_write(FILE *fp, GSList *prefs_filtering)
  *
  *\return	gboolean FALSE
  */
-static gboolean prefs_matcher_write_func(GNode *node, gpointer data)
+static gboolean prefs_matcher_write_func(GNode *node, gpointer d)
 {
 	FolderItem *item;
-	FILE *fp = data;
+	NodeLoopData *data = (NodeLoopData *)d;
 	gchar *id;
 	GSList *prefs_filtering;
 
@@ -2157,11 +2268,20 @@ static gboolean prefs_matcher_write_func(GNode *node, gpointer data)
         prefs_filtering = item->prefs->processing;
 
 	if (prefs_filtering != NULL) {
-		fprintf(fp, "[%s]\n", id);
-		prefs_filtering_write(fp, prefs_filtering);
-		fputc('\n', fp);
+		if (fprintf(data->fp, "[%s]\n", id) < 0) {
+			data->error = TRUE;
+			goto fail;
+		}
+		if (prefs_filtering_write(data->fp, prefs_filtering) < 0) {
+			data->error = TRUE;
+			goto fail;
+		}
+		if (fputc('\n', data->fp) == EOF) {
+			data->error = TRUE;
+			goto fail;
+		}
 	}
-
+fail:
 	g_free(id);
 
 	return FALSE;
@@ -2172,32 +2292,44 @@ static gboolean prefs_matcher_write_func(GNode *node, gpointer data)
  *
  *\param	fp File
  */
-static void prefs_matcher_save(FILE *fp)
+static int prefs_matcher_save(FILE *fp)
 {
 	GList *cur;
+	NodeLoopData data;
+	
+	data.fp = fp;
+	data.error = FALSE;
 
 	for (cur = folder_get_list() ; cur != NULL ; cur = g_list_next(cur)) {
 		Folder *folder;
 
 		folder = (Folder *) cur->data;
 		g_node_traverse(folder->node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
-				prefs_matcher_write_func, fp);
+				prefs_matcher_write_func, &data);
 	}
         
+	if (data.error == TRUE)
+		return -1;
+
         /* pre global rules */
-        fprintf(fp, "[preglobal]\n");
-        prefs_filtering_write(fp, pre_global_processing);
-        fputc('\n', fp);
+        if (fprintf(fp, "[preglobal]\n") < 0 ||
+            prefs_filtering_write(fp, pre_global_processing) < 0 ||
+            fputc('\n', fp) == EOF)
+		return -1;
 
         /* post global rules */
-        fprintf(fp, "[postglobal]\n");
-        prefs_filtering_write(fp, post_global_processing);
-        fputc('\n', fp);
+        if (fprintf(fp, "[postglobal]\n") < 0 ||
+            prefs_filtering_write(fp, post_global_processing) < 0 ||
+            fputc('\n', fp) == EOF)
+		return -1;
         
         /* filtering rules */
-        fprintf(fp, "[filtering]\n");
-        prefs_filtering_write(fp, filtering_rules);
-        fputc('\n', fp);
+	if (fprintf(fp, "[filtering]\n") < 0 ||
+            prefs_filtering_write(fp, filtering_rules) < 0 ||
+            fputc('\n', fp) == EOF)
+		return -1;
+
+	return 0;
 }
 
 /*!
@@ -2219,14 +2351,13 @@ void prefs_matcher_write_config(void)
 		return;
 	}
 
-
-	prefs_matcher_save(pfile->fp);
-
 	g_free(rcpath);
 
-	if (prefs_file_close(pfile) < 0) {
+	if (prefs_matcher_save(pfile->fp) < 0) {
 		g_warning("failed to write configuration to file\n");
-		return;
+		prefs_file_close_revert(pfile);
+	} else if (prefs_file_close(pfile) < 0) {
+		g_warning("failed to save configuration to file\n");
 	}
 }
 
@@ -2238,7 +2369,7 @@ static void matcher_add_rulenames(const gchar *rcpath)
 	FILE *src = g_fopen(rcpath, "rb");
 	FILE *dst = g_fopen(newpath, "wb");
 	gchar buf[BUFFSIZE];
-
+	int r;
 	if (dst == NULL) {
 		perror("fopen");
 		g_free(newpath);
@@ -2250,10 +2381,10 @@ static void matcher_add_rulenames(const gchar *rcpath)
 		&& strncmp(buf, "rulename \"", 10)
 		&& strncmp(buf, "enabled rulename \"", 18)
 		&& strncmp(buf, "disabled rulename \"", 18)) {
-			fwrite("enabled rulename \"\" ",
+			r = fwrite("enabled rulename \"\" ",
 				strlen("enabled rulename \"\" "), 1, dst);
 		}
-		fwrite(buf, strlen(buf), 1, dst);
+		r = fwrite(buf, strlen(buf), 1, dst);
 	}
 	fclose(dst);
 	fclose(src);
@@ -2297,7 +2428,7 @@ void prefs_matcher_read_config(void)
 	else {
 		/* previous version compatibility */
 
-		/* printf("reading filtering\n"); */
+		/* g_print("reading filtering\n"); */
 		rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
 				     FILTERING_RC, NULL);
 		f = g_fopen(rcpath, "rb");
@@ -2308,7 +2439,7 @@ void prefs_matcher_read_config(void)
 			fclose(matcher_parserin);
 		}
 		
-		/* printf("reading scoring\n"); */
+		/* g_print("reading scoring\n"); */
 		rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
 				     SCORING_RC, NULL);
 		f = g_fopen(rcpath, "rb");
