@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -128,6 +128,8 @@ static gchar   *mh_item_get_path		(Folder *folder,
 
 static gboolean mh_scan_required	(Folder		*folder,
 					 FolderItem	*item);
+static void mh_set_mtime		(Folder		*folder,
+					 FolderItem *item);
 static int mh_item_close		(Folder		*folder,
 					 FolderItem	*item);
 #if 0
@@ -160,6 +162,7 @@ FolderClass *mh_get_class(void)
 		mh_class.remove_folder = mh_remove_folder;
 		mh_class.get_num_list = mh_get_num_list;
 		mh_class.scan_required = mh_scan_required;
+		mh_class.set_mtime = mh_set_mtime;
 		mh_class.close = mh_item_close;
 		mh_class.get_flags = NULL; /*mh_get_flags */;
 
@@ -218,7 +221,7 @@ gboolean mh_scan_required(Folder *folder, FolderItem *item)
 	if ((s.st_mtime > item->mtime) &&
 		(s.st_mtime - 3600 != item->mtime)) {
 		debug_print("MH scan required, folder updated: %s (%ld > %ld)\n",
-			    path,
+			    path?path:"(null)",
 			    (long int) s.st_mtime,
 			    (long int) item->mtime);
 		g_free(path);
@@ -226,7 +229,7 @@ gboolean mh_scan_required(Folder *folder, FolderItem *item)
 	}
 
 	debug_print("MH scan not required: %s (%ld <= %ld)\n",
-		    path,
+		    path?path:"(null)",
 		    (long int) s.st_mtime,
 		    (long int) item->mtime);
 	g_free(path);
@@ -243,7 +246,7 @@ static void mh_get_last_num(Folder *folder, FolderItem *item)
 
 	g_return_if_fail(item != NULL);
 
-	debug_print("mh_get_last_num(): Scanning %s ...\n", item->path);
+	debug_print("mh_get_last_num(): Scanning %s ...\n", item->path?item->path:"(null)");
 
 	path = folder_item_get_path(item);
 	g_return_if_fail(path != NULL);
@@ -264,10 +267,12 @@ static void mh_get_last_num(Folder *folder, FolderItem *item)
 			if (max < num)
 				max = num;
 		}
+		if (num % 2000 == 0)
+			GTK_EVENTS_FLUSH();
 	}
 	closedir(dp);
 
-	debug_print("Last number in dir %s = %d\n", item->path, max);
+	debug_print("Last number in dir %s = %d\n", item->path?item->path:"(null)", max);
 	item->last_num = max;
 }
 
@@ -281,7 +286,7 @@ gint mh_get_num_list(Folder *folder, FolderItem *item, GSList **list, gboolean *
 
 	g_return_val_if_fail(item != NULL, -1);
 
-	debug_print("mh_get_num_list(): Scanning %s ...\n", item->path);
+	debug_print("mh_get_num_list(): Scanning %s ...\n", item->path?item->path:"(null)");
 
 	*old_uids_valid = TRUE;
 
@@ -306,7 +311,7 @@ gint mh_get_num_list(Folder *folder, FolderItem *item, GSList **list, gboolean *
 	}
 	closedir(dp);
 
-	mh_set_mtime(item);
+	mh_set_mtime(folder, item);
 	return nummsgs;
 }
 
@@ -454,6 +459,8 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, MsgInfoList *msglist,
 			 GRelation *relation)
 {
 	gboolean dest_need_scan = FALSE;
+	gboolean src_need_scan = FALSE;
+	FolderItem *src = NULL;
 	gchar *srcfile;
 	gchar *destfile;
 	gint filemode = 0;
@@ -463,7 +470,8 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, MsgInfoList *msglist,
 	gint curnum = 0, total = 0;
 	gchar *srcpath = NULL;
 	gboolean full_fetch = FALSE;
-	time_t last_mtime = (time_t)0;
+	time_t last_dest_mtime = (time_t)0;
+	time_t last_src_mtime = (time_t)0;
 
 	g_return_val_if_fail(dest != NULL, -1);
 	g_return_val_if_fail(msglist != NULL, -1);
@@ -479,6 +487,10 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, MsgInfoList *msglist,
 
 	if (msginfo->folder->folder != dest->folder)
 		full_fetch = TRUE;
+	
+	if (FOLDER_TYPE(msginfo->folder->folder) == F_MH) {
+		src = msginfo->folder;
+	}
 
 	if (dest->last_num < 0) {
 		mh_get_last_num(folder, dest);
@@ -490,7 +502,12 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, MsgInfoList *msglist,
 	srcpath = folder_item_get_path(msginfo->folder);
 
 	dest_need_scan = mh_scan_required(dest->folder, dest);
-	last_mtime = dest->mtime;
+	last_dest_mtime = dest->mtime;
+
+	if (src) {
+		src_need_scan = mh_scan_required(src->folder, src);
+		last_src_mtime = src->mtime;
+	}
 
 	total = g_slist_length(msglist);
 	if (total > 100) {
@@ -571,9 +588,14 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, MsgInfoList *msglist,
 	g_free(srcpath);
 	mh_write_sequences(dest, TRUE);
 
-	if (dest->mtime == last_mtime && !dest_need_scan) {
-		mh_set_mtime(dest);
+	if (dest->mtime == last_dest_mtime && !dest_need_scan) {
+		mh_set_mtime(folder, dest);
 	}
+
+	if (src && src->mtime == last_src_mtime && !src_need_scan) {
+		mh_set_mtime(folder, src);
+	}
+
 	if (total > 100) {
 		statusbar_progress_all(0,0,0);
 		statusbar_pop_all();
@@ -611,7 +633,7 @@ static gint mh_remove_msg(Folder *folder, FolderItem *item, gint num)
 	}
 
 	if (item->mtime == last_mtime && !need_scan) {
-		mh_set_mtime(item);
+		mh_set_mtime(folder, item);
 	}
 	g_free(file);
 	return 0;
@@ -670,7 +692,7 @@ static gint mh_remove_msgs(Folder *folder, FolderItem *item,
 		statusbar_pop_all();
 	}
 	if (item->mtime == last_mtime && !need_scan) {
-		mh_set_mtime(item);
+		mh_set_mtime(folder, item);
 	}
 
 	g_free(path);
@@ -978,7 +1000,7 @@ static gboolean mh_remove_missing_folder_items_func(GNode *node, gpointer data)
 
 	path = folder_item_get_path(item);
 	if (!is_dir_exist(path)) {
-		debug_print("folder '%s' not found. removing...\n", path);
+		debug_print("folder '%s' not found. removing...\n", path?path:"(null)");
 		folder_item_remove(item);
 	}
 	g_free(path);
@@ -1122,7 +1144,7 @@ static void mh_scan_tree_recursive(FolderItem *item)
 	closedir(dp);
 #endif
 
-	mh_set_mtime(item);
+	mh_set_mtime(folder, item);
 }
 
 static gboolean mh_rename_folder_func(GNode *node, gpointer data)
@@ -1308,7 +1330,7 @@ next_token:
 	}
 /*
 	g_timer_stop(timer);
-	printf("mh_get_flags: %f secs\n", g_timer_elapsed(timer, NULL));
+	g_print("mh_get_flags: %f secs\n", g_timer_elapsed(timer, NULL));
 	g_timer_destroy(timer);
 */
 	return 0;
@@ -1321,6 +1343,7 @@ static void mh_write_sequences(FolderItem *item, gboolean remove_unseen)
 	FILE *mh_sequences_old_fp, *mh_sequences_new_fp;
 	gchar buf[BUFFSIZE];
 	gchar *path = NULL;
+	gboolean err = FALSE;
 	START_TIMING("");
 
 	if (!item)
@@ -1370,16 +1393,21 @@ static void mh_write_sequences(FolderItem *item, gboolean remove_unseen)
 			cur = cur ? cur->next:NULL;
 		} while (cur || (start > 0 && end > 0));
 		if (sequence && *sequence) {
-			fprintf(mh_sequences_new_fp, "%s%s\n", 
-					get_unseen_seq_name(), sequence);
-			debug_print("wrote unseen sequence: '%s%s'\n", 
+			if (fprintf(mh_sequences_new_fp, "%s%s\n", 
+					get_unseen_seq_name(), sequence) < 0)
+				err = TRUE;
+			else
+				debug_print("wrote unseen sequence: '%s%s'\n", 
 					get_unseen_seq_name(), sequence);
 		}
 		/* rewrite the rest of the file */
 		if ((mh_sequences_old_fp = g_fopen(mh_sequences_old, "r+b")) != NULL) {
 			while (fgets(buf, sizeof(buf), mh_sequences_old_fp) != NULL) {
 				if (strncmp(buf, get_unseen_seq_name(), strlen(get_unseen_seq_name())))
-					fprintf(mh_sequences_new_fp, "%s", buf);
+					if (fprintf(mh_sequences_new_fp, "%s", buf) < 0) {
+						err = TRUE;
+						break;
+					}
 			}
 			fclose(mh_sequences_old_fp);
 		}
@@ -1388,9 +1416,11 @@ static void mh_write_sequences(FolderItem *item, gboolean remove_unseen)
 #if 0
 		fsync(fileno(mh_sequences_new_fp));
 #endif
-		fclose(mh_sequences_new_fp);
+		if (fclose(mh_sequences_new_fp) == EOF)
+			err = TRUE;
 
-		g_rename(mh_sequences_new, mh_sequences_old);
+		if (!err)
+			g_rename(mh_sequences_new, mh_sequences_old);
 		g_free(sequence);
 		procmsg_msg_list_free(msglist);
 	}
@@ -1410,13 +1440,13 @@ static int mh_item_close(Folder *folder, FolderItem *item)
 	mh_write_sequences(item, FALSE);
 
 	if (item->mtime == last_mtime && !need_scan) {
-		mh_set_mtime(item);
+		mh_set_mtime(folder, item);
 	}
 
 	return 0;
 }
 
-void mh_set_mtime(FolderItem *item)
+static void mh_set_mtime(Folder *folder, FolderItem *item)
 {
 	struct stat s;
 	gchar *path = folder_item_get_path(item);
@@ -1430,6 +1460,6 @@ void mh_set_mtime(FolderItem *item)
 	}
 
 	item->mtime = s.st_mtime;
-	debug_print("MH: forced mtime of %s to %ld\n", item->name, item->mtime);
+	debug_print("MH: forced mtime of %s to %ld\n", item->name?item->name:"(null)", item->mtime);
 	g_free(path);
 }
