@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,7 +28,12 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
-#include <sys/mman.h>
+#ifdef _WIN32
+# include <w32lib.h>
+# define MAP_FAILED	((char *) -1)
+#else
+# include <sys/mman.h>
+#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 
@@ -39,6 +44,7 @@
 #include "procmsg.h"
 #include "codeconv.h"
 #include "timing.h"
+#include "tags.h"
 
 #ifdef HAVE_FWRITE_UNLOCKED
 #define SC_FWRITE fwrite_unlocked
@@ -162,7 +168,7 @@ void msgcache_add_msg(MsgCache *cache, MsgInfo *msginfo)
 	cache->memusage += procmsg_msginfo_memusage(msginfo);
 	cache->last_access = time(NULL);
 
-	debug_print("Cache size: %d messages, %d bytes\n", g_hash_table_size(cache->msgnum_table), cache->memusage);
+	debug_print("Cache size: %d messages, %u bytes\n", g_hash_table_size(cache->msgnum_table), cache->memusage);
 }
 
 void msgcache_remove_msg(MsgCache *cache, guint msgnum)
@@ -183,7 +189,7 @@ void msgcache_remove_msg(MsgCache *cache, guint msgnum)
 	procmsg_msginfo_free(msginfo);
 	cache->last_access = time(NULL);
 
-	debug_print("Cache size: %d messages, %d byte\n", g_hash_table_size(cache->msgnum_table), cache->memusage);
+	debug_print("Cache size: %d messages, %u bytes\n", g_hash_table_size(cache->msgnum_table), cache->memusage);
 }
 
 void msgcache_update_msg(MsgCache *cache, MsgInfo *msginfo)
@@ -209,7 +215,7 @@ void msgcache_update_msg(MsgCache *cache, MsgInfo *msginfo)
 	cache->memusage += procmsg_msginfo_memusage(newmsginfo);
 	cache->last_access = time(NULL);
 	
-	debug_print("Cache size: %d messages, %d byte\n", g_hash_table_size(cache->msgnum_table), cache->memusage);
+	debug_print("Cache size: %d messages, %u bytes\n", g_hash_table_size(cache->msgnum_table), cache->memusage);
 
 	return;
 }
@@ -299,7 +305,7 @@ gint msgcache_get_memory_usage(MsgCache *cache)
 	size_t ni; \
  \
 	if ((ni = fread(&idata, sizeof(idata), 1, fp)) != 1) { \
-		g_warning("read_int: Cache data corrupted, read %d of %d at " \
+		g_warning("read_int: Cache data corrupted, read %zd of %zd at " \
 			  "offset %ld\n", ni, sizeof(idata), ftell(fp)); \
 		procmsg_msginfo_free(msginfo); \
 		error = TRUE; \
@@ -318,7 +324,7 @@ gint msgcache_get_memory_usage(MsgCache *cache)
 { \
 	GET_CACHE_DATA_INT(tmp_len);	\
 	if ((tmp_len = msgcache_get_cache_data_str(walk_data, &data, tmp_len, conv)) < 0) { \
-		printf("error at rem_len:%d\n", rem_len);\
+		g_print("error at rem_len:%d\n", rem_len);\
 		procmsg_msginfo_free(msginfo); \
 		error = TRUE; \
 		goto bail_err; \
@@ -389,7 +395,7 @@ static FILE *msgcache_open_data_file(const gchar *file, guint version,
 
 	if (mode == DATA_WRITE) {
 		int w_err = 0, wrote = 0;
-		if ((fp = g_fopen(file, "w+")) == NULL) {
+		if ((fp = g_fopen(file, "wb")) == NULL) {
 			FILE_OP_ERROR(file, "fopen");
 			return NULL;
 		}
@@ -450,16 +456,16 @@ static gint msgcache_read_cache_data_str(FILE *fp, gchar **str,
 	if (!swapping) {
 		if ((ni = fread(&len, sizeof(len), 1, fp) != 1) ||
 		    len > G_MAXINT) {
-			g_warning("read_data_str: Cache data (len) corrupted, read %d "
-				  "of %d bytes at offset %ld\n", ni, sizeof(len), 
+			g_warning("read_data_str: Cache data (len) corrupted, read %zd "
+				  "of %zd bytes at offset %ld\n", ni, sizeof(len), 
 				  ftell(fp));
 			return -1;
 		}
 	} else {
 		if ((ni = fread(&len, sizeof(len), 1, fp) != 1) ||
 		    bswap_32(len) > G_MAXINT) {
-			g_warning("read_data_str: Cache data (len) corrupted, read %d "
-				  "of %d bytes at offset %ld\n", ni, sizeof(len), 
+			g_warning("read_data_str: Cache data (len) corrupted, read %zd "
+				  "of %zd bytes at offset %ld\n", ni, sizeof(len), 
 				  ftell(fp));
 			return -1;
 		}
@@ -472,12 +478,11 @@ static gint msgcache_read_cache_data_str(FILE *fp, gchar **str,
 	tmpstr = g_try_malloc(len + 1);
 
 	if(tmpstr == NULL) {
-		g_warning("read_data_str: can't g_malloc %d bytes - cache data probably corrupted.\n", len);
 		return -1;
 	}
 
 	if ((ni = fread(tmpstr, 1, len, fp)) != len) {
-		g_warning("read_data_str: Cache data corrupted, read %d of %d "
+		g_warning("read_data_str: Cache data corrupted, read %zd of %u "
 			  "bytes at offset %ld\n", 
 			  ni, len, ftell(fp));
 		g_free(tmpstr);
@@ -512,7 +517,6 @@ static gint msgcache_get_cache_data_str(gchar *src, gchar **str, gint len,
 	tmpstr = g_try_malloc(len + 1);
 
 	if(tmpstr == NULL) {
-		g_warning("read_data_str: can't g_malloc %d bytes - cache data probably corrupted.\n", len);
 		return -1;
 	}
 
@@ -590,8 +594,10 @@ MsgCache *msgcache_read_cache(FolderItem *item, const gchar *cache_file)
 		tmp_flags |= MSG_DRAFT;
 	}
 
-	if (msgcache_read_cache_data_str(fp, &srccharset, NULL) < 0)
+	if (msgcache_read_cache_data_str(fp, &srccharset, NULL) < 0) {
+		fclose(fp);
 		return NULL;
+	}
 	dstcharset = CS_UTF_8;
 	if (srccharset == NULL || dstcharset == NULL) {
 		conv = NULL;
@@ -621,8 +627,24 @@ MsgCache *msgcache_read_cache(FolderItem *item, const gchar *cache_file)
 			map_len = st.st_size;
 		else
 			map_len = -1;
-		if (map_len > 0)
+		if (map_len > 0) {
+#ifdef G_OS_WIN32
+			cache_data = NULL;
+			HANDLE hFile, hMapping;
+			hFile = (HANDLE) _get_osfhandle (fileno(fp));
+			if (hFile == (HANDLE) -1)
+				goto w32_fail;
+			hMapping = CreateFileMapping(hFile, NULL, PAGE_WRITECOPY, 0, 0, NULL);
+			if (!hMapping)
+				goto w32_fail;
+			cache_data = (unsigned char *)MapViewOfFile(hMapping, FILE_MAP_COPY, 0, 0, 0);
+			CloseHandle (hMapping);
+		w32_fail:
+			;
+#else
 			cache_data = mmap(NULL, map_len, PROT_READ, MAP_PRIVATE, fileno(fp), 0);
+#endif
+		}
 	} else {
 		cache_data = NULL;
 	}
@@ -678,8 +700,12 @@ MsgCache *msgcache_read_cache(FolderItem *item, const gchar *cache_file)
 			if(msginfo->msgid)
 				g_hash_table_insert(cache->msgid_table, msginfo->msgid, msginfo);
 		}
-		
+
+#ifdef G_OS_WIN32
+		UnmapViewOfFile((void*) cache_data);
+#else
 		munmap(cache_data, map_len);
+#endif
 	} else {
 		while (fread(&num, sizeof(num), 1, fp) == 1) {
 			if (swapping)
@@ -749,7 +775,7 @@ bail_err:
 	cache->memusage = memusage;
 
 	debug_print("done. (%d items read)\n", g_hash_table_size(cache->msgnum_table));
-	debug_print("Cache size: %d messages, %d byte\n", g_hash_table_size(cache->msgnum_table), cache->memusage);
+	debug_print("Cache size: %d messages, %u bytes\n", g_hash_table_size(cache->msgnum_table), cache->memusage);
 	return cache;
 }
 
@@ -784,8 +810,24 @@ void msgcache_read_mark(MsgCache *cache, const gchar *mark_file)
 			map_len = st.st_size;
 		else
 			map_len = -1;
-		if (map_len > 0)
+		if (map_len > 0) {
+#ifdef G_OS_WIN32
+			cache_data = NULL;
+			HANDLE hFile, hMapping;
+			hFile = (HANDLE) _get_osfhandle (fileno(fp));
+			if (hFile == (HANDLE) -1)
+				goto w32_fail2;
+			hMapping = CreateFileMapping(hFile, NULL, PAGE_WRITECOPY, 0, 0, NULL);
+			if (!hMapping)
+				goto w32_fail2;
+			cache_data = (unsigned char *)MapViewOfFile(hMapping, FILE_MAP_COPY, 0, 0, 0);
+			CloseHandle (hMapping);
+		w32_fail2:
+			;
+#else
 			cache_data = mmap(NULL, map_len, PROT_READ, MAP_PRIVATE, fileno(fp), 0);
+#endif
+		}
 	} else {
 		cache_data = NULL;
 	}
@@ -801,7 +843,11 @@ void msgcache_read_mark(MsgCache *cache, const gchar *mark_file)
 				msginfo->flags.perm_flags = perm_flags;
 			}
 		}
+#ifdef G_OS_WIN32
+		UnmapViewOfFile((void*) cache_data);
+#else
 		munmap(cache_data, map_len);
+#endif
 	} else {
 		while (fread(&num, sizeof(num), 1, fp) == 1) {
 			if (swapping)
@@ -818,6 +864,111 @@ void msgcache_read_mark(MsgCache *cache, const gchar *mark_file)
 	fclose(fp);
 }
 
+void msgcache_read_tags(MsgCache *cache, const gchar *tags_file)
+{
+	FILE *fp;
+	MsgInfo *msginfo;
+	guint32 num;
+	gint map_len = -1;
+	char *cache_data = NULL;
+	struct stat st;
+	
+	swapping = TRUE;
+
+	/* In case we can't open the mark file with MARK_VERSION, check if we can open it with the
+	 * swapped MARK_VERSION. As msgcache_open_data_file swaps it too, if this succeeds, 
+	 * it means it's the old version (not little-endian) on a big-endian machine. The code has
+	 * no effect on x86 as their file doesn't change. */
+
+	if ((fp = msgcache_open_data_file(tags_file, TAGS_VERSION, DATA_READ, NULL, 0)) == NULL) {
+		/* see if it isn't swapped ? */
+		if ((fp = msgcache_open_data_file(tags_file, bswap_32(TAGS_VERSION), DATA_READ, NULL, 0)) == NULL)
+			return;
+		else
+			swapping = FALSE; /* yay */
+	}
+	debug_print("reading %sswapped tags file.\n", swapping?"":"un");
+	
+	if (msgcache_use_mmap_read) {
+		if (fstat(fileno(fp), &st) >= 0)
+			map_len = st.st_size;
+		else
+			map_len = -1;
+		if (map_len > 0) {
+#ifdef G_OS_WIN32
+			cache_data = NULL;
+			HANDLE hFile, hMapping;
+			hFile = (HANDLE) _get_osfhandle (fileno(fp));
+			if (hFile == (HANDLE) -1)
+				goto w32_fail6;
+			hMapping = CreateFileMapping(hFile, NULL, PAGE_WRITECOPY, 0, 0, NULL);
+			if (!hMapping)
+				goto w32_fail6;
+			cache_data = (unsigned char *)MapViewOfFile(hMapping, FILE_MAP_COPY, 0, 0, 0);
+			CloseHandle (hMapping);
+		w32_fail6:
+			;
+#else
+			cache_data = mmap(NULL, map_len, PROT_READ, MAP_PRIVATE, fileno(fp), 0);
+#endif
+		}
+	} else {
+		cache_data = NULL;
+	}
+	if (cache_data != NULL && cache_data != MAP_FAILED) {
+		int rem_len = map_len-ftell(fp);
+		char *walk_data = cache_data+ftell(fp);
+
+		while(rem_len > 0) {
+			gint id = -1;
+			GET_CACHE_DATA_INT(num);
+			msginfo = g_hash_table_lookup(cache->msgnum_table, &num);
+			if(msginfo) {
+				g_slist_free(msginfo->tags);
+				msginfo->tags = NULL;
+				do {
+					GET_CACHE_DATA_INT(id);
+					if (id > 0) {
+						msginfo->tags = g_slist_prepend(
+							msginfo->tags, 
+							GINT_TO_POINTER(id));
+					}
+				} while (id > 0);
+				msginfo->tags = g_slist_reverse(msginfo->tags);
+			}
+		}
+#ifdef G_OS_WIN32
+		UnmapViewOfFile((void*) cache_data);
+#else
+		munmap(cache_data, map_len);
+#endif
+	} else {
+		while (fread(&num, sizeof(num), 1, fp) == 1) {
+			gint id = -1;
+			if (swapping)
+				num = bswap_32(num);
+			msginfo = g_hash_table_lookup(cache->msgnum_table, &num);
+			if(msginfo) {
+				g_slist_free(msginfo->tags);
+				msginfo->tags = NULL;
+				do {
+					if (fread(&id, sizeof(id), 1, fp) != 1) 
+						id = -1;
+					if (swapping)
+						id = bswap_32(id);
+					if (id > 0) {
+						msginfo->tags = g_slist_prepend(
+							msginfo->tags, 
+							GINT_TO_POINTER(id));
+					}
+				} while (id > 0);
+				msginfo->tags = g_slist_reverse(msginfo->tags);
+			}
+		}
+	}
+	fclose(fp);
+}
+
 static int msgcache_write_cache(MsgInfo *msginfo, FILE *fp)
 {
 	MsgTmpFlags flags = msginfo->flags.tmp_flags & MSG_CACHED_FLAG_MASK;
@@ -894,6 +1045,23 @@ static int msgcache_write_flags(MsgInfo *msginfo, FILE *fp)
 	return w_err ? -1 : wrote;
 }
 
+static int msgcache_write_tags(MsgInfo *msginfo, FILE *fp)
+{
+	GSList *cur = msginfo->tags;
+	int w_err = 0, wrote = 0;
+
+	WRITE_CACHE_DATA_INT(msginfo->msgnum, fp);
+	for (; cur; cur = cur->next) {
+		gint id = GPOINTER_TO_INT(cur->data);
+		if (tags_get_tag(id) != NULL) {
+			WRITE_CACHE_DATA_INT(id, fp);
+		}
+	}
+	WRITE_CACHE_DATA_INT(-1, fp);
+
+	return w_err ? -1 : wrote;
+}
+
 static int msgcache_write_mmap_flags(MsgInfo *msginfo, char *walk_data)
 {
 	MsgPermFlags flags = msginfo->flags.perm_flags;
@@ -904,15 +1072,34 @@ static int msgcache_write_mmap_flags(MsgInfo *msginfo, char *walk_data)
 	return wrote;
 }
 
+static int msgcache_write_mmap_tags(MsgInfo *msginfo, char *walk_data)
+{
+	GSList *cur = msginfo->tags;
+	int wrote = 0;
+	
+	PUT_CACHE_DATA_INT(msginfo->msgnum);
+	for (; cur; cur = cur->next) {
+		gint id = GPOINTER_TO_INT(cur->data);
+		if (tags_get_tag(id) != NULL) {
+			PUT_CACHE_DATA_INT(id);
+		}
+	}
+	PUT_CACHE_DATA_INT(-1);
+	return wrote;
+}
+
 struct write_fps
 {
 	FILE *cache_fp;
 	FILE *mark_fp;
+	FILE *tags_fp;
 	char *cache_data;
 	char *mark_data;
+	char *tags_data;
 	int error;
 	guint cache_size;
 	guint mark_size;
+	guint tags_size;
 };
 
 static void msgcache_write_func(gpointer key, gpointer value, gpointer user_data)
@@ -934,6 +1121,11 @@ static void msgcache_write_func(gpointer key, gpointer value, gpointer user_data
 		write_fps->error = 1;
 	else
 		write_fps->mark_size += tmp;
+	tmp = msgcache_write_tags(msginfo, write_fps->tags_fp);
+	if (tmp < 0)
+		write_fps->error = 1;
+	else
+		write_fps->tags_size += tmp;
 }
 
 static void msgcache_write_mmap_func(gpointer key, gpointer value, gpointer user_data)
@@ -951,32 +1143,42 @@ static void msgcache_write_mmap_func(gpointer key, gpointer value, gpointer user
 	tmp = msgcache_write_mmap_flags(msginfo, write_fps->mark_data);
 	write_fps->mark_size += tmp;
 	write_fps->mark_data += tmp;
+	tmp = msgcache_write_mmap_tags(msginfo, write_fps->tags_data);
+	write_fps->tags_size += tmp;
+	write_fps->tags_data += tmp;
 }
 
-gint msgcache_write(const gchar *cache_file, const gchar *mark_file, MsgCache *cache)
+gint msgcache_write(const gchar *cache_file, const gchar *mark_file, const gchar *tags_file, MsgCache *cache)
 {
 	struct write_fps write_fps;
-	gchar *new_cache, *new_mark;
+	gchar *new_cache, *new_mark, *new_tags;
 	int w_err = 0, wrote = 0;
 	gint map_len = -1;
 	char *cache_data = NULL;
 	char *mark_data = NULL;
+	char *tags_data = NULL;
+
 	START_TIMING("");
 	g_return_val_if_fail(cache_file != NULL, -1);
 	g_return_val_if_fail(mark_file != NULL, -1);
+	g_return_val_if_fail(tags_file != NULL, -1);
 	g_return_val_if_fail(cache != NULL, -1);
 
 	new_cache = g_strconcat(cache_file, ".new", NULL);
 	new_mark  = g_strconcat(mark_file, ".new", NULL);
+	new_tags  = g_strconcat(tags_file, ".new", NULL);
 
 	write_fps.error = 0;
 	write_fps.cache_size = 0;
 	write_fps.mark_size = 0;
+	write_fps.tags_size = 0;
+
 	write_fps.cache_fp = msgcache_open_data_file(new_cache, CACHE_VERSION,
 		DATA_WRITE, NULL, 0);
 	if (write_fps.cache_fp == NULL) {
 		g_free(new_cache);
 		g_free(new_mark);
+		g_free(new_tags);
 		return -1;
 	}
 
@@ -988,6 +1190,7 @@ gint msgcache_write(const gchar *cache_file, const gchar *mark_file, MsgCache *c
 		g_unlink(new_cache);
 		g_free(new_cache);
 		g_free(new_mark);
+		g_free(new_tags);
 		return -1;
 	}
 
@@ -998,6 +1201,20 @@ gint msgcache_write(const gchar *cache_file, const gchar *mark_file, MsgCache *c
 		g_unlink(new_cache);
 		g_free(new_cache);
 		g_free(new_mark);
+		g_free(new_tags);
+		return -1;
+	}
+
+	write_fps.tags_fp = msgcache_open_data_file(new_tags, TAGS_VERSION,
+		DATA_WRITE, NULL, 0);
+	if (write_fps.tags_fp == NULL) {
+		fclose(write_fps.cache_fp);
+		fclose(write_fps.mark_fp);
+		g_unlink(new_cache);
+		g_unlink(new_mark);
+		g_free(new_cache);
+		g_free(new_mark);
+		g_free(new_tags);
 		return -1;
 	}
 
@@ -1008,20 +1225,88 @@ gint msgcache_write(const gchar *cache_file, const gchar *mark_file, MsgCache *c
 
 	write_fps.cache_size = ftell(write_fps.cache_fp);
 	write_fps.mark_size = ftell(write_fps.mark_fp);
+	write_fps.tags_size = ftell(write_fps.tags_fp);
+
 	if (msgcache_use_mmap_write && cache->memusage > 0) {
 		map_len = cache->memusage;
 		if (ftruncate(fileno(write_fps.cache_fp), (off_t)map_len) == 0) {
+
+#ifdef G_OS_WIN32
+			cache_data = NULL;
+			HANDLE hFile, hMapping;
+			hFile = (HANDLE) _get_osfhandle (fileno(write_fps.cache_fp));
+			if (hFile == (HANDLE) -1)
+				goto w32_fail3;
+			hMapping = CreateFileMapping(hFile, NULL, PAGE_READWRITE, 0, 0, NULL);
+			if (!hMapping)
+				goto w32_fail3;
+			cache_data = (unsigned char *)MapViewOfFile(hMapping, FILE_MAP_COPY, 0, 0, 0);
+			CloseHandle (hMapping);
+		w32_fail3:
+			;
+#else
 			cache_data = mmap(NULL, map_len, PROT_WRITE, MAP_SHARED, 
 				fileno(write_fps.cache_fp), 0);
+#endif
 		}
 		if (cache_data != NULL && cache_data != MAP_FAILED) {
 			if (ftruncate(fileno(write_fps.mark_fp), (off_t)map_len) == 0) {
+#ifdef G_OS_WIN32
+				mark_data = NULL;
+				HANDLE hFile, hMapping;
+				hFile = (HANDLE) _get_osfhandle (fileno(write_fps.mark_fp));
+				if (hFile == (HANDLE) -1)
+					goto w32_fail4;
+				hMapping = CreateFileMapping(hFile, NULL, PAGE_READWRITE, 0, 0, NULL);
+				if (!hMapping)
+					goto w32_fail4;
+				mark_data = (unsigned char *)MapViewOfFile(hMapping, FILE_MAP_COPY, 0, 0, 0);
+				CloseHandle (hMapping);
+			w32_fail4:
+				;
+#else
 				mark_data = mmap(NULL, map_len, PROT_WRITE, MAP_SHARED, 
 					fileno(write_fps.mark_fp), 0);
+#endif
 			} 
 			if (mark_data == NULL || mark_data == MAP_FAILED) {
+#ifdef G_OS_WIN32
+				UnmapViewOfFile((void*) cache_data);
+#else
 				munmap(cache_data, map_len);
+#endif
 				cache_data = NULL;
+			} else {
+				if (ftruncate(fileno(write_fps.tags_fp), (off_t)map_len) == 0) {
+#ifdef G_OS_WIN32
+						tags_data = NULL;
+						HANDLE hFile, hMapping;
+						hFile = (HANDLE) _get_osfhandle (fileno(write_fps.tags_fp));
+						if (hFile == (HANDLE) -1)
+							goto w32_fail5;
+						hMapping = CreateFileMapping(hFile, NULL, PAGE_READWRITE, 0, 0, NULL);
+						if (!hMapping)
+							goto w32_fail5;
+						tags_data = (unsigned char *)MapViewOfFile(hMapping, FILE_MAP_COPY, 0, 0, 0);
+						CloseHandle (hMapping);
+					w32_fail5:
+						;
+#else
+					tags_data = mmap(NULL, map_len, PROT_WRITE, MAP_SHARED, 
+						fileno(write_fps.tags_fp), 0);
+#endif
+				} 
+				if (tags_data == NULL || tags_data == MAP_FAILED) {
+#ifdef G_OS_WIN32
+					UnmapViewOfFile((void*) cache_data);
+					UnmapViewOfFile((void*) mark_data);
+#else
+					munmap(cache_data, map_len);
+					munmap(mark_data, map_len);
+#endif
+					cache_data = NULL;
+					mark_data = NULL;
+				} 
 			}
 		}
 	}
@@ -1029,49 +1314,67 @@ gint msgcache_write(const gchar *cache_file, const gchar *mark_file, MsgCache *c
 	if (cache_data != NULL && cache_data != MAP_FAILED) {
 		write_fps.cache_data = cache_data + ftell(write_fps.cache_fp);
 		write_fps.mark_data = mark_data + ftell(write_fps.mark_fp);
+		write_fps.tags_data = mark_data + ftell(write_fps.tags_fp);
 		g_hash_table_foreach(cache->msgnum_table, msgcache_write_mmap_func, (gpointer)&write_fps);
+#ifdef G_OS_WIN32
+		UnmapViewOfFile((void*) cache_data);
+		UnmapViewOfFile((void*) mark_data);
+		UnmapViewOfFile((void*) tags_data);
+#else
 		munmap(cache_data, map_len);
 		munmap(mark_data, map_len);
+		munmap(tags_data, map_len);
+#endif
 		ftruncate(fileno(write_fps.cache_fp), write_fps.cache_size);
 		ftruncate(fileno(write_fps.mark_fp), write_fps.mark_size);
+		ftruncate(fileno(write_fps.tags_fp), write_fps.tags_size);
 	} else {
 #ifdef HAVE_FWRITE_UNLOCKED
 		flockfile(write_fps.cache_fp);
 		flockfile(write_fps.mark_fp);
+		flockfile(write_fps.tags_fp);
 #endif
 		g_hash_table_foreach(cache->msgnum_table, msgcache_write_func, (gpointer)&write_fps);
 #ifdef HAVE_FWRITE_UNLOCKED
 		funlockfile(write_fps.mark_fp);
 		funlockfile(write_fps.cache_fp);
+		funlockfile(write_fps.tags_fp);
 #endif
 	}
 	
 	fflush(write_fps.cache_fp);
 	fflush(write_fps.mark_fp);
+	fflush(write_fps.tags_fp);
 
 #if 0
 	fsync(fileno(write_fps.cache_fp));
 	fsync(fileno(write_fps.mark_fp));
+	fsync(fileno(write_fps.tags_fp));
 #endif
 
 	fclose(write_fps.cache_fp);
 	fclose(write_fps.mark_fp);
+	fclose(write_fps.tags_fp);
 
 
 	if (write_fps.error != 0) {
 		g_unlink(new_cache);
 		g_unlink(new_mark);
+		g_unlink(new_tags);
 		g_free(new_cache);
 		g_free(new_mark);
+		g_free(new_tags);
 		return -1;
 	} else {
 		move_file(new_cache, cache_file, TRUE);
 		move_file(new_mark, mark_file, TRUE);
+		move_file(new_tags, tags_file, TRUE);
 		cache->last_access = time(NULL);
 	}
 
 	g_free(new_cache);
 	g_free(new_mark);
+	g_free(new_tags);
 	debug_print("done.\n");
 	END_TIMING();
 	return 0;
