@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,6 +49,7 @@
 #include "log.h"
 #include "prefs_common.h"
 #include "alertpanel.h"
+#include "addr_compl.h"
 
 #ifdef HAVE_SYSEXITS_H
 #include <sysexits.h>
@@ -110,6 +111,10 @@ static PrefParam param[] = {
 	 NULL, NULL, NULL},
 	{"mark_as_read", "TRUE", &config.mark_as_read, P_BOOL,
 	 NULL, NULL, NULL},
+	{"whitelist_ab", "FALSE", &config.whitelist_ab, P_BOOL,
+	 NULL, NULL, NULL},
+	{"whitelist_ab_folder", N_("Any"), &config.whitelist_ab_folder, P_STRING,
+	 NULL, NULL, NULL},
 
 	{NULL, NULL, NULL, P_OTHER, NULL, NULL, NULL}
 };
@@ -190,6 +195,33 @@ static MsgStatus msg_is_spam(FILE *fp)
 	return is_spam ? MSG_IS_SPAM:MSG_IS_HAM;
 }
 
+static gboolean sa_found_in_addressbook(const gchar *address)
+{
+	gchar *addr = NULL;
+	gboolean found = FALSE;
+	gint num_addr = 0;
+	
+	if (!address)
+		return FALSE;
+	
+	addr = g_strdup(address);
+	extract_address(addr);
+	num_addr = complete_address(addr);
+	if (num_addr > 1) {
+		/* skip first item (this is the search string itself) */
+		int i = 1;
+		for (; i < num_addr && !found; i++) {
+			gchar *caddr = get_complete_address(i);
+			extract_address(caddr);
+			if (strcasecmp(caddr, addr) == 0)
+				found = TRUE;
+			g_free(caddr);
+		}
+	}
+	g_free(addr);
+	return found;
+}
+
 static gboolean mail_filtering_hook(gpointer source, gpointer data)
 {
 	MailFilteringData *mail_filtering_data = (MailFilteringData *) source;
@@ -214,6 +246,30 @@ static gboolean mail_filtering_hook(gpointer source, gpointer data)
 		return FALSE;
 	}
 
+	if (config.whitelist_ab) {
+		gchar *ab_folderpath;
+		gboolean whitelisted = FALSE;
+
+		if (*config.whitelist_ab_folder == '\0' ||
+			strcasecmp(config.whitelist_ab_folder, _("Any")) == 0) {
+			/* match the whole addressbook */
+			ab_folderpath = NULL;
+		} else {
+			/* match the specific book/folder of the addressbook */
+			ab_folderpath = config.whitelist_ab_folder;
+		}
+
+		start_address_completion(ab_folderpath);
+		if (msginfo->from && 
+		    sa_found_in_addressbook(msginfo->from))
+				whitelisted = TRUE;
+		end_address_completion();
+		
+		if (whitelisted) {
+			debug_print("message is ham (whitelisted)\n");
+			return FALSE;
+		}
+	}
 	pid = fork();
 	if (pid == 0) {
 		_exit(msg_is_spam(fp));
@@ -298,7 +354,7 @@ static gboolean mail_filtering_hook(gpointer source, gpointer data)
 			if (config.mark_as_read)
 				procmsg_msginfo_unset_flags(msginfo, ~0, 0);
 			procmsg_msginfo_set_flags(msginfo, MSG_SPAM, 0);
-			msginfo->is_move = TRUE;
+			msginfo->filter_op = IS_MOVE;
 			msginfo->to_filter_folder = save_folder;
 		} else {
 			folder_item_remove_msg(msginfo->folder, msginfo->msgnum);
@@ -317,13 +373,11 @@ static gboolean mail_filtering_hook(gpointer source, gpointer data)
 					   "sure spamd is running and accessible.");
 		if (!prefs_common.no_recv_err_panel) {
 			if (!warned_error) {
-				alertpanel_error(msg);
+				alertpanel_error("%s", msg);
 			}
 			warned_error = TRUE;
 		} else {
-			gchar *tmp = g_strdup_printf("%s\n", msg);
-			log_error(LOG_PROTOCOL, tmp);
-			g_free(tmp);
+			log_error(LOG_PROTOCOL, "%s\n", msg);
 		}
 	}
 	
@@ -472,9 +526,11 @@ void spamassassin_save_config(void)
 		prefs_file_close_revert(pfile);
 		return;
 	}
-	fprintf(pfile->fp, "\n");
-
-	prefs_file_close(pfile);
+        if (fprintf(pfile->fp, "\n") < 0) {
+		FILE_OP_ERROR(rcpath, "fprintf");
+		prefs_file_close_revert(pfile);
+	} else
+	        prefs_file_close(pfile);
 }
 
 gboolean spamassassin_check_username(void)
@@ -579,7 +635,7 @@ const gchar *plugin_type(void)
 
 const gchar *plugin_licence(void)
 {
-	return "GPL";
+	return "GPL3+";
 }
 
 const gchar *plugin_version(void)
