@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -50,6 +50,7 @@
 #include "addr_compl.h"
 #include "manual.h"
 #include "combobox.h"
+#include "menu.h"
 
 #include "matcher_parser.h"
 #include "matcher.h"
@@ -99,9 +100,14 @@ static void prefs_filtering_set_list	(void);
 static void prefs_filtering_register_cb	(void);
 static void prefs_filtering_substitute_cb	(void);
 static void prefs_filtering_delete_cb	(void);
+static void prefs_filtering_delete_all_cb(void);
+static void prefs_filtering_clear_cb(void);
+static void prefs_filtering_duplicate_cb(void);
 static void prefs_filtering_top		(void);
+static void prefs_filtering_page_up(void);
 static void prefs_filtering_up		(void);
 static void prefs_filtering_down	(void);
+static void prefs_filtering_page_down(void);
 static void prefs_filtering_bottom	(void);
 static gint prefs_filtering_deleted	(GtkWidget	*widget,
 					 GdkEventAny	*event,
@@ -117,6 +123,7 @@ static void prefs_filtering_action_define(void);
 static gint prefs_filtering_list_view_set_row	(gint row, FilteringProp * prop);
 					  
 static void prefs_filtering_reset_dialog	(void);
+static gboolean prefs_filtering_rename_tag_func(GNode *node, gpointer data);
 static gboolean prefs_filtering_rename_path_func(GNode *node, gpointer data);
 static gboolean prefs_filtering_delete_path_func(GNode *node, gpointer data);
 
@@ -143,16 +150,14 @@ static void prefs_filtering_list_view_get_rule_info	(GtkWidget *list,
 static GtkWidget *prefs_filtering_list_view_create	(void);
 static void prefs_filtering_create_list_view_columns	(GtkWidget *list_view);
 
-static gboolean prefs_filtering_selected		(GtkTreeSelection *selector,
-							 GtkTreeModel *model, 
-							 GtkTreePath *path,
-							 gboolean currently_selected,
-							 gpointer data);
+static void prefs_filtering_select_row(GtkTreeView *list_view, GtkTreePath *path);
 
 static void prefs_filtering_account_option_menu_populate(void);
 
 static gulong signal_id = 0; /* filtering.help_btn clicked signal */
 
+static int modified = FALSE;
+
 void prefs_filtering_open(GSList ** p_processing,
 			  const gchar *title,
 			  const gchar *help_url_anchor,
@@ -229,8 +234,13 @@ static void prefs_filtering_size_allocate_cb(GtkWidget *widget,
 /* prefs_filtering_close() - just to have one common exit point */
 static void prefs_filtering_close(void)
 {
+	GtkListStore *store;
+
 	end_address_completion();
-	
+
+	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW
+				(filtering.cond_list_view)));
+	gtk_list_store_clear(store);
 	gtk_widget_hide(filtering.window);
 	inc_unlock();
 }
@@ -298,22 +308,29 @@ static void prefs_filtering_create(void)
 	GtkWidget *reg_btn;
 	GtkWidget *subst_btn;
 	GtkWidget *del_btn;
+	GtkWidget *clear_btn;
 
 	GtkWidget *cond_hbox;
 	GtkWidget *cond_scrolledwin;
 	GtkWidget *cond_list_view;
 
 	GtkWidget *btn_vbox;
-	GtkWidget *spc_vbox;
 	GtkWidget *top_btn;
 	GtkWidget *up_btn;
 	GtkWidget *down_btn;
+#ifndef MAEMO
+	GtkWidget *page_up_btn;
+	GtkWidget *page_down_btn;
+#endif
 	GtkWidget *bottom_btn;
 	GtkWidget *table;
 	static GdkGeometry geometry;
+	GtkTooltips *tooltips;
 
 	debug_print("Creating filtering configuration window...\n");
 
+	tooltips = gtk_tooltips_new();
+
 	window = gtkut_window_new(GTK_WINDOW_TOPLEVEL, "prefs_filtering");
 	gtk_container_set_border_width (GTK_CONTAINER (window), 8);
 	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
@@ -448,6 +465,8 @@ static void prefs_filtering_create(void)
 	gtk_box_pack_start (GTK_BOX (btn_hbox), reg_btn, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT (reg_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_register_cb), NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), reg_btn,
+			_("Append the new rule above to the list"), NULL);
 
 	subst_btn = gtkut_get_replace_btn (_("Replace"));
 	gtk_widget_show (subst_btn);
@@ -455,12 +474,24 @@ static void prefs_filtering_create(void)
 	g_signal_connect(G_OBJECT (subst_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_substitute_cb),
 			 NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), subst_btn,
+			_("Replace the selected rule in list with the rule above"), NULL);
 
 	del_btn = gtk_button_new_from_stock (GTK_STOCK_DELETE);
 	gtk_widget_show (del_btn);
 	gtk_box_pack_start (GTK_BOX (btn_hbox), del_btn, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT (del_btn), "clicked",
 			G_CALLBACK(prefs_filtering_delete_cb), NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), del_btn,
+			_("Delete the selected rule from the list"), NULL);
+
+	clear_btn = gtk_button_new_from_stock (GTK_STOCK_CLEAR);
+	gtk_widget_show (clear_btn);
+	gtk_box_pack_start (GTK_BOX (btn_hbox), clear_btn, FALSE, TRUE, 0);
+	g_signal_connect(G_OBJECT (clear_btn), "clicked",
+			G_CALLBACK(prefs_filtering_clear_cb), NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), clear_btn,
+			_("Clear all the input fields in the dialog"), NULL);
 
 	cond_hbox = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (cond_hbox);
@@ -488,32 +519,60 @@ static void prefs_filtering_create(void)
 	gtk_box_pack_start (GTK_BOX (btn_vbox), top_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (top_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_top), NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), top_btn,
+			_("Move the selected rule to the top"), NULL);
 
-	PACK_VSPACER (btn_vbox, spc_vbox, VSPACING_NARROW_2);
+#ifndef MAEMO
+	page_up_btn = gtk_button_new_with_mnemonic (_("Page up"));
+	gtk_button_set_image(GTK_BUTTON(page_up_btn),
+			gtk_image_new_from_stock(GTK_STOCK_GO_UP,GTK_ICON_SIZE_BUTTON));
+	gtk_widget_show (page_up_btn);
+	gtk_box_pack_start (GTK_BOX (btn_vbox), page_up_btn, FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT (page_up_btn), "clicked",
+			 G_CALLBACK(prefs_filtering_page_up), NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), page_up_btn,
+			_("Move the selected rule one page up"), NULL);
+#endif
 
 	up_btn = gtk_button_new_from_stock (GTK_STOCK_GO_UP);
 	gtk_widget_show (up_btn);
 	gtk_box_pack_start (GTK_BOX (btn_vbox), up_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (up_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_up), NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), up_btn,
+			_("Move the selected rule up"), NULL);
 
 	down_btn = gtk_button_new_from_stock (GTK_STOCK_GO_DOWN);
 	gtk_widget_show (down_btn);
 	gtk_box_pack_start (GTK_BOX (btn_vbox), down_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (down_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_down), NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), down_btn,
+			_("Move the selected rule down"), NULL);
 
-	PACK_VSPACER (btn_vbox, spc_vbox, VSPACING_NARROW_2);
+#ifndef MAEMO
+	page_down_btn = gtk_button_new_with_mnemonic (_("Page down"));
+	gtk_button_set_image(GTK_BUTTON(page_down_btn),
+			gtk_image_new_from_stock(GTK_STOCK_GO_DOWN,GTK_ICON_SIZE_BUTTON));
+	gtk_widget_show (page_down_btn);
+	gtk_box_pack_start (GTK_BOX (btn_vbox), page_down_btn, FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT (page_down_btn), "clicked",
+			 G_CALLBACK(prefs_filtering_page_down), NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), page_down_btn,
+			_("Move the selected rule one page down"), NULL);
+#endif
 
 	bottom_btn = gtk_button_new_from_stock (GTK_STOCK_GOTO_BOTTOM);
 	gtk_widget_show (bottom_btn);
 	gtk_box_pack_start (GTK_BOX (btn_vbox), bottom_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (bottom_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_bottom), NULL);
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), bottom_btn,
+			_("Move the selected rule to the bottom"), NULL);
 
 	if (!geometry.min_height) {
 		geometry.min_width = 500;
-		geometry.min_height = 400;
+		geometry.min_height = 460;
 	}
 
 	gtk_window_set_geometry_hints(GTK_WINDOW(window), NULL, &geometry,
@@ -534,6 +593,30 @@ static void prefs_filtering_create(void)
 	filtering.account_label  = account_label;
 }
 
+static void rename_tag(GSList * filters,
+			const gchar * old_tag, const gchar * new_tag);
+
+void prefs_filtering_rename_tag(const gchar *old_tag, const gchar *new_tag)
+{
+	GList * cur;
+	const gchar *tags[2] = {NULL, NULL};
+	tags[0] = old_tag;
+	tags[1] = new_tag;
+	for (cur = folder_get_list() ; cur != NULL ; cur = g_list_next(cur)) {
+		Folder *folder;
+		folder = (Folder *) cur->data;
+		g_node_traverse(folder->node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
+				prefs_filtering_rename_tag_func, tags);
+	}
+        
+	rename_tag(pre_global_processing, old_tag, new_tag);
+	rename_tag(post_global_processing, old_tag, new_tag);
+	rename_tag(filtering_rules, old_tag, new_tag);
+        
+	prefs_matcher_write_config();
+}
+
+
 static void rename_path(GSList * filters,
 			const gchar * old_path, const gchar * new_path);
 
@@ -582,7 +665,11 @@ static void rename_path(GSList * filters,
 
                         FilteringAction *action = action_cur->data;
                         
-                        if (!action->destination) continue;
+                        if (action->type == MATCHACTION_SET_TAG ||
+			    action->type == MATCHACTION_UNSET_TAG)
+				continue;
+                        if (!action->destination) 
+				continue;
                         
                         destlen = strlen(action->destination);
                         
@@ -660,6 +747,59 @@ static gboolean prefs_filtering_rename_path_func(GNode *node, gpointer data)
 	return FALSE;
 }
 
+static void rename_tag(GSList * filters,
+			const gchar * old_tag, const gchar * new_tag)
+{
+        GSList * action_cur;
+        GSList * cur;
+
+	for (cur = filters; cur != NULL; cur = cur->next) {
+		FilteringProp   *filtering = (FilteringProp *)cur->data;
+                
+                for(action_cur = filtering->action_list ; action_cur != NULL ;
+                    action_cur = action_cur->next) {
+
+                        FilteringAction *action = action_cur->data;
+                        
+                        if (action->type != MATCHACTION_SET_TAG &&
+			    action->type != MATCHACTION_UNSET_TAG)
+				continue;
+                        if (!action->destination)
+				continue;
+                        if (!strcmp(action->destination, old_tag)) {
+				g_free(action->destination);
+				action->destination = g_strdup(new_tag);
+			}
+                }
+        }
+}
+
+static gboolean prefs_filtering_rename_tag_func(GNode *node, gpointer data)
+{
+	GSList *filters;
+	const gchar * old_tag;
+        const gchar * new_tag;
+        const gchar ** tags;
+	FolderItem *item;
+        
+        tags = data;
+	old_tag = tags[0];
+	new_tag = tags[1];
+
+	g_return_val_if_fail(old_tag != NULL, FALSE);
+	g_return_val_if_fail(new_tag != NULL, FALSE);
+	g_return_val_if_fail(node != NULL, FALSE);
+
+        item = node->data;
+        if (!item || !item->prefs)
+                return FALSE;
+        filters = item->prefs->processing;
+
+        rename_tag(filters, old_tag, new_tag);
+
+	return FALSE;
+}
+
 void prefs_filtering_delete_path(const gchar *path)
 {
 	GList * cur;
@@ -700,7 +840,11 @@ static void delete_path(GSList ** p_filters, const gchar * path)
                         
                         action = action_cur->data;
                         
-                        if (!action->destination) continue;
+                        if (action->type == MATCHACTION_SET_TAG ||
+			    action->type == MATCHACTION_UNSET_TAG)
+				continue;
+                        if (!action->destination) 
+				continue;
                         
                         destlen = strlen(action->destination);
                         
@@ -742,15 +886,8 @@ static gboolean prefs_filtering_delete_path_func(GNode *node, gpointer data)
 	return FALSE;
 }
 
-static void prefs_filtering_set_dialog(const gchar *header, const gchar *key)
+static void prefs_filtering_clear_list(GtkListStore *list_store)
 {
-	GtkTreeView *list_view = GTK_TREE_VIEW(filtering.cond_list_view);
-	GSList *cur;
-	GSList * prefs_filtering;
-	gchar *cond_str;
-	GtkListStore *list_store;
-	
-	list_store = GTK_LIST_STORE(gtk_tree_view_get_model(list_view));
 	gtk_list_store_clear(list_store);
 
 	/* add the place holder (New) at row 0 */
@@ -761,6 +898,18 @@ static void prefs_filtering_set_dialog(const gchar *header, const gchar *key)
 					      _("(New)"),
 					      _("(New)"),
 					      FALSE);
+}
+
+static void prefs_filtering_set_dialog(const gchar *header, const gchar *key)
+{
+	GtkTreeView *list_view = GTK_TREE_VIEW(filtering.cond_list_view);
+	GSList *cur;
+	GSList * prefs_filtering;
+	gchar *cond_str;
+	GtkListStore *list_store;
+	
+	list_store = GTK_LIST_STORE(gtk_tree_view_get_model(list_view));
+	prefs_filtering_clear_list(list_store);
 
         prefs_filtering = *p_processing_list;
 
@@ -1031,7 +1180,7 @@ fail:
 
 static void prefs_filtering_register_cb(void)
 {
-	FilteringProp * prop;
+	FilteringProp *prop;
 	
 	prop = prefs_filtering_dialog_to_filtering(TRUE);
 	if (prop == NULL)
@@ -1041,6 +1190,7 @@ static void prefs_filtering_register_cb(void)
 	filteringprop_free(prop);
 
 	prefs_filtering_reset_dialog();
+	modified = TRUE;
 }
 
 static void prefs_filtering_substitute_cb(void)
@@ -1072,6 +1222,7 @@ static void prefs_filtering_substitute_cb(void)
 	filteringprop_free(prop);
 
 	prefs_filtering_reset_dialog();
+	modified = TRUE;
 }
 
 static void prefs_filtering_delete_cb(void)
@@ -1079,10 +1230,10 @@ static void prefs_filtering_delete_cb(void)
 	GtkTreeView *list_view = GTK_TREE_VIEW(filtering.cond_list_view);
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	gint row;
+	gint selected_row;
 	
-	row = gtkut_list_view_get_selected_row(filtering.cond_list_view);
-	if (row <= 0) 
+	selected_row = gtkut_list_view_get_selected_row(filtering.cond_list_view);
+	if (selected_row <= 0) 
 		return;	
 
 	if (alertpanel(_("Delete rule"),
@@ -1091,12 +1242,65 @@ static void prefs_filtering_delete_cb(void)
 		return;
 
 	model = gtk_tree_view_get_model(list_view);	
-	if (!gtk_tree_model_iter_nth_child(model, &iter, NULL, row))
+	if (!gtk_tree_model_iter_nth_child(model, &iter, NULL, selected_row))
 		return;
 
 	gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
 
 	prefs_filtering_reset_dialog();
+	modified = TRUE;
+}
+
+static void prefs_filtering_delete_all_cb(void)
+{
+	GtkListStore *list_store;
+	
+	if (alertpanel(_("Delete all rules"),
+		       _("Do you really want to delete all the rules?"),
+		       GTK_STOCK_CANCEL, "+"GTK_STOCK_DELETE, NULL) == G_ALERTDEFAULT)
+		return;
+
+	list_store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(filtering.cond_list_view)));
+	prefs_filtering_clear_list(list_store);
+
+	prefs_filtering_reset_dialog();
+	modified = TRUE;
+}
+
+static void prefs_filtering_clear_cb(void)
+{
+	prefs_filtering_reset_dialog();
+}
+
+static void prefs_filtering_duplicate_cb(void)
+{
+	gint selected_row = gtkut_list_view_get_selected_row
+		(filtering.cond_list_view);	
+	FilteringProp *prop;
+	gboolean enabled;
+	gchar *name;
+	gint account_id;
+
+	if (selected_row <= 0) 
+		return;	
+	
+	prop = prefs_filtering_dialog_to_filtering(TRUE);
+	if (prop == NULL)
+		return;
+
+	/* prop->emabled is always TRUE here, re-use the value from the selected row 
+	   as we don't substitute this value from dialog */
+	prefs_filtering_list_view_get_rule_info(
+			filtering.cond_list_view, selected_row,
+			&enabled, &name, &account_id);
+	prop->enabled = enabled;
+
+	prefs_filtering_list_view_set_row(-selected_row-2, prop);
+	
+	filteringprop_free(prop);
+
+	prefs_filtering_reset_dialog();
+	modified = TRUE;
 }
 
 static void prefs_filtering_top(void)
@@ -1117,8 +1321,41 @@ static void prefs_filtering_top(void)
 
 	gtk_list_store_move_after(GTK_LIST_STORE(model), &sel, &top);
 	gtkut_list_view_select_row(filtering.cond_list_view, 1);
+	modified = TRUE;
 }
 
+static void prefs_filtering_page_up(void)
+{
+	gint row, target_row, n_rows;
+	GtkTreeIter selected, target;
+	GtkTreeModel *model;
+	GtkTreePath *path;
+	GdkRectangle cell_rect, view_rect;
+
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(filtering.cond_list_view));	
+	n_rows = gtk_tree_model_iter_n_children(model, NULL);
+	row = gtkut_list_view_get_selected_row(filtering.cond_list_view);
+	if (row <= 1)
+		return;
+
+	if (!gtk_tree_model_iter_nth_child(model, &selected, NULL, row))
+		return;
+
+	/* compute number of rows per page (approximation) */
+	path = gtk_tree_model_get_path(model, &selected);
+	gtk_tree_view_get_cell_area(GTK_TREE_VIEW(filtering.cond_list_view), path, NULL, &cell_rect);
+	gtk_tree_view_get_visible_rect(GTK_TREE_VIEW(filtering.cond_list_view), &view_rect);
+	gtk_tree_path_free(path);
+	target_row = row - (view_rect.height/cell_rect.height);
+	if (target_row < 1)
+		target_row = 1;
+
+	if (!gtk_tree_model_iter_nth_child(model, &target, NULL, target_row))
+		return;
+	gtk_list_store_move_before(GTK_LIST_STORE(model), &selected, &target);
+	gtkut_list_view_select_row(filtering.cond_list_view, target_row);
+	modified = TRUE;
+}
 static void prefs_filtering_up(void)
 {
 	gint row;
@@ -1137,6 +1374,7 @@ static void prefs_filtering_up(void)
 
 	gtk_list_store_swap(GTK_LIST_STORE(model), &top, &sel);
 	gtkut_list_view_select_row(filtering.cond_list_view, row - 1);
+	modified = TRUE;
 }
 
 static void prefs_filtering_down(void)
@@ -1157,6 +1395,40 @@ static void prefs_filtering_down(void)
 			
 	gtk_list_store_swap(GTK_LIST_STORE(model), &top, &sel);
 	gtkut_list_view_select_row(filtering.cond_list_view, row + 1);
+	modified = TRUE;
+}
+
+static void prefs_filtering_page_down(void)
+{
+	gint row, target_row, n_rows;
+	GtkTreeIter selected, target;
+	GtkTreeModel *model;
+	GtkTreePath *path;
+	GdkRectangle cell_rect, view_rect;
+
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(filtering.cond_list_view));	
+	n_rows = gtk_tree_model_iter_n_children(model, NULL);
+	row = gtkut_list_view_get_selected_row(filtering.cond_list_view);
+	if (row < 1 || row >= n_rows -1)
+		return;
+
+	if (!gtk_tree_model_iter_nth_child(model, &selected, NULL, row))
+		return;
+
+	/* compute number of rows per page (approximation) */
+	path = gtk_tree_model_get_path(model, &selected);
+	gtk_tree_view_get_cell_area(GTK_TREE_VIEW(filtering.cond_list_view), path, NULL, &cell_rect);
+	gtk_tree_view_get_visible_rect(GTK_TREE_VIEW(filtering.cond_list_view), &view_rect);
+	gtk_tree_path_free(path);
+	target_row = row + (view_rect.height/cell_rect.height);
+	if (target_row > n_rows-1)
+		target_row = n_rows-1;
+
+	if (!gtk_tree_model_iter_nth_child(model, &target, NULL, target_row))
+		return;
+	gtk_list_store_move_after(GTK_LIST_STORE(model), &selected, &target);
+	gtkut_list_view_select_row(filtering.cond_list_view, target_row);
+	modified = TRUE;
 }
 
 static void prefs_filtering_bottom(void)
@@ -1177,6 +1449,7 @@ static void prefs_filtering_bottom(void)
 
 	gtk_list_store_move_after(GTK_LIST_STORE(model), &top, &sel);		
 	gtkut_list_view_select_row(filtering.cond_list_view, n_rows - 1);
+	modified = TRUE;
 }
 
 static void prefs_filtering_select_set(FilteringProp *prop)
@@ -1225,7 +1498,7 @@ static gboolean prefs_filtering_key_pressed(GtkWidget *widget, GdkEventKey *even
 	return FALSE;
 }
 
-static void prefs_filtering_ok(void)
+static gboolean prefs_filtering_check_mod(gboolean check_changed_list)
 {
 	FilteringProp * prop;
 	gchar * str;
@@ -1234,6 +1507,17 @@ static void prefs_filtering_ok(void)
         AlertValue val;
 	
 	prop = prefs_filtering_dialog_to_filtering(FALSE);
+	
+	if (check_changed_list) {
+		if (modified && alertpanel(_("Filtering rules not saved"),
+					 _("The list of filtering rules have been modified. Close anyway?"),
+					 GTK_STOCK_CLOSE, _("+_Continue editing"), 
+					 NULL) != G_ALERTDEFAULT) {
+			return TRUE;
+		}
+	}
+	
+	/* check if a rule is being edited */
 	if (prop != NULL) {
 		str = filteringprop_to_string(prop);
 
@@ -1254,7 +1538,7 @@ static void prefs_filtering_ok(void)
 				g_free(filtering_str);
 				g_free(str); /* fixed two leaks: huzzah! */
 				filteringprop_free(prop);
-				return;
+				return TRUE;
 			}
 		}		
 
@@ -1276,13 +1560,21 @@ static void prefs_filtering_ok(void)
 				g_free(name);
 				g_free(condition);
 				g_free(action);
-				return;
+				return TRUE;
 			}
 		}
 		g_free(name);
 		g_free(condition);
 		g_free(action);
 	}
+	return FALSE;
+}
+
+static void prefs_filtering_ok(void)
+{
+	if (prefs_filtering_check_mod(FALSE))
+		return;
+	modified = FALSE;
 	prefs_filtering_set_list();
 	prefs_matcher_write_config();
 	prefs_filtering_close();
@@ -1290,6 +1582,9 @@ static void prefs_filtering_ok(void)
 
 static void prefs_filtering_cancel(void)
 {
+	if (prefs_filtering_check_mod(TRUE))
+		return;
+	modified = FALSE;
 	prefs_matcher_read_config();
 	prefs_filtering_close();
 }
@@ -1312,8 +1607,9 @@ static GtkListStore* prefs_filtering_create_data_store(void)
  *		at least it made GTK 2 porting easier.
  *
  *\param	list_store Store to operate on
- *\param	row -1 to add a new rule to store, else change an existing
- *		row
+ *\param	row -1 to add a new rule to store,
+			row >=0 to change an existing row
+			row <-1 insert a new row after (-row-2)
  *\param	enabled TRUE if rule is enabled
  *\param	name The Name of rule
  *\param	account_id The account ID
@@ -1334,15 +1630,20 @@ static gint prefs_filtering_list_view_insert_rule(GtkListStore *list_store,
 						  gboolean prop) 
 {
 	GtkTreeIter iter;
+	GtkTreeIter sibling;
 
 	/* check if valid row at all */
 	if (row >= 0) {
 		if (!gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(list_store),
 						   &iter, NULL, row))
+			row = -1;
+	} else if (row < -1) {
+		if (!gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(list_store),
+						   &sibling, NULL, -row-2))
 			row = -1;						   
 	}
 
-	if (row < 0) {
+	if (row == -1 ) {
 		/* append new */
 		gtk_list_store_append(list_store, &iter);
 		gtk_list_store_set(list_store, &iter, 
@@ -1355,6 +1656,19 @@ static gint prefs_filtering_list_view_insert_rule(GtkListStore *list_store,
 				   -1);
 		return gtk_tree_model_iter_n_children(GTK_TREE_MODEL(list_store),
 						      NULL) - 1;
+	} else if (row < -1) {
+		/* duplicate */
+		gtk_list_store_insert_after(list_store, &iter, &sibling);
+		gtk_list_store_set(list_store, &iter, 
+				   PREFS_FILTERING_ENABLED, enabled,
+				   PREFS_FILTERING_NAME, name,
+				   PREFS_FILTERING_ACCOUNT_ID, account_id,
+				   PREFS_FILTERING_ACCOUNT_NAME, account_name,
+				   PREFS_FILTERING_RULE, rule,
+				   PREFS_FILTERING_PROP, prop,
+				   -1);
+		return gtk_tree_model_iter_n_children(GTK_TREE_MODEL(list_store),
+						      NULL) - 1;
 	} else {
 		/* change existing */
 		gtk_list_store_set(list_store, &iter, 
@@ -1407,6 +1721,85 @@ static void prefs_filtering_list_view_get_rule_info(GtkWidget *list, gint row,
 	}
 }
 
+static GtkItemFactory *prefs_filtering_popup_factory = NULL;
+static GtkWidget *prefs_filtering_popup_menu = NULL;
+
+static GtkItemFactoryEntry prefs_filtering_popup_entries[] =
+{
+	{N_("/_Delete"),		NULL, prefs_filtering_delete_cb, 0, NULL, NULL},
+	{N_("/Delete _all"),		NULL, prefs_filtering_delete_all_cb, 0, NULL, NULL},
+	{N_("/D_uplicate"),		NULL, prefs_filtering_duplicate_cb, 0, NULL, NULL},
+#ifdef MAEMO
+	{N_("/---"),			NULL, NULL, 0, "<Separator>", NULL},
+	{N_("/Move one page up"),		NULL, prefs_filtering_page_up, 0, NULL, NULL},
+	{N_("/Move one page down"),		NULL, prefs_filtering_page_down, 0, NULL, NULL},
+#endif
+};
+
+static gint prefs_filtering_list_btn_pressed(GtkWidget *widget, GdkEventButton *event,
+				    GtkTreeView *list_view)
+{
+	if (event) {
+		/* left- or right-button click */
+		if (event->button == 1 || event->button == 3) {
+			GtkTreePath *path = NULL;
+
+			if (gtk_tree_view_get_path_at_pos( list_view, event->x, event->y,
+								&path, NULL, NULL, NULL)) {
+				prefs_filtering_select_row(list_view, path);
+			}
+			if (path)
+				gtk_tree_path_free(path);
+		}
+
+		/* right-button click */
+		if (event->button == 3) {
+			GtkTreeModel *model = gtk_tree_view_get_model(list_view);
+			GtkTreeIter iter;
+			gboolean non_empty;
+			gint row;
+
+			if (!prefs_filtering_popup_menu) {
+				gint n_entries = sizeof(prefs_filtering_popup_entries) /
+						sizeof(prefs_filtering_popup_entries[0]);
+				prefs_filtering_popup_menu = menu_create_items(prefs_filtering_popup_entries,
+								  n_entries, "<PrefsFilteringPopupMenu>",
+								  &prefs_filtering_popup_factory, list_view);
+			}
+
+			/* grey out some popup menu items if there is no selected row */
+			row = gtkut_list_view_get_selected_row(GTK_WIDGET(list_view));
+			menu_set_sensitive(prefs_filtering_popup_factory, "/Delete", (row > 0));
+			menu_set_sensitive(prefs_filtering_popup_factory, "/Duplicate", (row > 0));
+
+			/* grey out seom popup menu items if there is no row
+			   (not counting the (New) one at row 0) */
+			non_empty = gtk_tree_model_get_iter_first(model, &iter);
+			if (non_empty)
+				non_empty = gtk_tree_model_iter_next(model, &iter);
+			menu_set_sensitive(prefs_filtering_popup_factory, "/Delete all", non_empty);
+
+			gtk_menu_popup(GTK_MENU(prefs_filtering_popup_menu), 
+			    	   NULL, NULL, NULL, NULL, 
+			    	   event->button, event->time);
+		}
+	}
+	return FALSE;
+}
+
+static gboolean prefs_filtering_list_popup_menu(GtkWidget *widget, gpointer data)
+{
+	GtkTreeView *list_view = (GtkTreeView *)data;
+	GdkEventButton event;
+	
+	event.button = 3;
+	event.time = gtk_get_current_event_time();
+	
+	prefs_filtering_list_btn_pressed(NULL, &event, list_view);
+
+	return TRUE;
+}
+
 /*!
  *\brief	Create list view for filtering
  */
@@ -1417,14 +1810,27 @@ static GtkWidget *prefs_filtering_list_view_create(void)
 
 	list_view = GTK_TREE_VIEW(gtk_tree_view_new_with_model(GTK_TREE_MODEL
 		(prefs_filtering_create_data_store())));
+#ifdef MAEMO
+	g_object_set(list_view, "allow-checkbox-mode", FALSE, NULL);
+#endif
+
+#ifndef MAEMO
+	g_signal_connect(G_OBJECT(list_view), "popup-menu",
+			 G_CALLBACK(prefs_filtering_list_popup_menu), list_view);
+#else
+	gtk_widget_tap_and_hold_setup(GTK_WIDGET(list_view), NULL, NULL,
+			GTK_TAP_AND_HOLD_NONE | GTK_TAP_AND_HOLD_NO_INTERNALS);
+	g_signal_connect(G_OBJECT(list_view), "tap-and-hold",
+			 G_CALLBACK(prefs_filtering_list_popup_menu), list_view);
+#endif
+	g_signal_connect(G_OBJECT(list_view), "button-press-event",
+			G_CALLBACK(prefs_filtering_list_btn_pressed), list_view);
 	
 	gtk_tree_view_set_rules_hint(list_view, prefs_common.use_stripes_everywhere);
 	gtk_tree_view_set_reorderable(list_view, TRUE);
 	
 	selector = gtk_tree_view_get_selection(list_view);
 	gtk_tree_selection_set_mode(selector, GTK_SELECTION_BROWSE);
-	gtk_tree_selection_set_select_function(selector, prefs_filtering_selected,
-					       NULL, NULL);
 
 	/* create the columns */
 	prefs_filtering_create_list_view_columns(GTK_WIDGET(list_view));
@@ -1503,20 +1909,22 @@ static void prefs_filtering_create_list_view_columns(GtkWidget *list_view)
 }
 
 /*!
- *\brief	Triggered when a row is selected
+ *\brief	Triggered when a row has to be selected
  */
-static gboolean prefs_filtering_selected(GtkTreeSelection *selector,
-					 GtkTreeModel *model, 
-					 GtkTreePath *path,
-					 gboolean currently_selected,
-					 gpointer data)
+static void prefs_filtering_select_row(GtkTreeView *list_view, GtkTreePath *path)
 {
-	if (currently_selected)
-		return TRUE;
-	else {		
-		gboolean has_prop  = FALSE;
+	GtkTreeModel *model = gtk_tree_view_get_model(list_view);
+
+	if (path && model) {
+		GtkTreeSelection *selection;
+		gboolean has_prop = FALSE;
 		GtkTreeIter iter;
 
+		/* select row */
+		selection = gtk_tree_view_get_selection(list_view);
+		gtk_tree_selection_select_path(selection, path);
+
+		/* update dialog from selection */
 		gtk_tree_model_get_iter(model, &iter, path);
 		gtk_tree_model_get(model, &iter,
 				   PREFS_FILTERING_PROP, &has_prop,
@@ -1549,8 +1957,5 @@ static gboolean prefs_filtering_selected(GtkTreeSelection *selector,
 			g_free(filtering_str);
 		} else
 			prefs_filtering_reset_dialog();
-	}		
-
-	return TRUE;
+	}
 }
-
