@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,14 +14,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ * 
  */
 
 #ifndef __PREFS_TEMPLATES_H__
 #define __PREFS_TEMPLATES_H__
 
 void prefs_template_open(void);
-gboolean prefs_template_string_is_valid(gchar *string, gint *line);
+gboolean prefs_template_string_is_valid(gchar *string, gint *line, gboolean escaped_string);
 
 #endif /* __PREFS_TEMPLATES_H__ */
